@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -209,9 +208,10 @@ bool Basic_buffer::pvalid( Point & p ) const
 
 
 // insert block b.[p1,p2) at p and move next
-bool Basic_buffer::pputb( Point & p, const Basic_buffer & b, const Point & p1, const Point & p2 )
+bool Basic_buffer::pputb( Point & p, const Basic_buffer & b,
+                          const Point & p1, const Point & p2 )
   {
-  if( !pisvalid( p ) || !b.pisvalid( p1 ) || !b.pisvalid( p2 ) || p1 >= p2 )
+  if( !pisvalid( p ) || p1 >= p2 || !b.pisvalid( p1 ) || !b.pisvalid( p2 ) )
     return false;
   const int lines = p2.line - p1.line;
   if( lines == 0 )
@@ -357,16 +357,16 @@ bool Basic_buffer::piseow( const Point & p ) const
 Point Basic_buffer::bol( const Point & p ) const
   {
   if( p.line >= lines() ) return eof();
-  else if( p.line < 0 ) return bof();
-  else return Point( p.line, 0 );
+  if( p.line < 0 ) return bof();
+  return Point( p.line, 0 );
   }
 
 
 Point Basic_buffer::eol( const Point & p ) const
   {
   if( p.line >= last_line() ) return eof();
-  else if( p.line < 0 ) return eol( bof() );
-  else return Point( p.line, data[p.line].size() - 1 );
+  if( p.line < 0 ) return eol( bof() );
+  return Point( p.line, data[p.line].size() - 1 );
   }
 
 
