@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -56,6 +55,29 @@ std::vector< Point * > & get_marks_and_pointers( Buffer & b )
   return v;
   }
 
+
+     // adjust marks and pointers after a block insert/delete/replace
+void adjust_marks_and_pointers( Buffer & b, const Point & p1,
+                                const Point & p2_old, const Point & p2_new,
+                                const Point * const p_ignore = 0 )
+  {
+  std::vector< Point * > & pv = get_marks_and_pointers( b );
+  for( unsigned i = 0; i < pv.size(); ++i )
+    if( *pv[i] > p1 && pv[i] != p_ignore )
+      {
+      if( *pv[i] < p2_old )			// was in the block, keep in
+        {
+        if( *pv[i] >= p2_new ) { *pv[i] = p2_new; b.pprev( *pv[i] ); }
+        else if( !b.pisvalid( *pv[i] ) ) b.pvalid( *pv[i] );
+        }
+      else					// was beyond the block
+        {
+        if( pv[i]->line == p2_old.line ) pv[i]->col += p2_new.col - p2_old.col;
+        pv[i]->line += p2_new.line - p2_old.line;
+        }
+      }
+  }
+
 } // end namespace
 
 
@@ -154,16 +176,20 @@ bool Buffer::Options::set_rmargin( const int r )
   }
 
 
-Buffer::Buffer( const Options & opts, const std::string * const namep )
-  : backuped( false ), modified_( false ), saved( false ),
-    options( opts ), crlf( false )
+Buffer::Buffer( const std::string * const namep )
+  : backuped( false ), modified_( false ), saved( false ), crlf( false )
   {
   FILE * f = 0;
   errno = 0;
   if( ( !namep || namep->empty() || *namep == "-" ) && !isatty( fileno( stdin ) ) )
     f = stdin;
   else if( namep && namep->size() )
-    { name_ = *namep; f = std::fopen( name_.c_str(), "r" ); }
+    {
+    name_ = *namep;
+    f = std::fopen( name_.c_str(), "r+" );
+    if( !f && errno != ENOENT )
+      { f = std::fopen( name_.c_str(), "r" ); options.read_only = true; }
+    }
   else return;
 
   if( !f )
@@ -189,11 +215,7 @@ Buffer::Buffer( const Options & opts, const std::string * const namep )
 
   const bool file_error = std::ferror( f );
   if( file_eof ) file_eof = std::feof( f );
-  if( f != stdin )
-    {
-    if( std::freopen( name_.c_str(), "r+", f ) ) std::fclose( f );
-    else options.read_only = true;
-    }
+  if( f != stdin ) std::fclose( f );
   else if( !ttyname( 1 ) || !std::freopen( ttyname( 1 ), "r", stdin ) )
     throw Error( "Error opening terminal for input" );
   if( file_error ) throw Error( "Error reading file" );
@@ -208,7 +230,7 @@ int Buffer::set_marks()
   int c = 0;
   if( mark[0].line < 0 )
     {
-    for( int i = 10; i < 60 && i < lines(); ++i )
+    for( int i = 10; i < 80 && i < lines(); ++i )
       if( get_line( i ).compare( 0, 10, "#include \"" ) == 0 )
         { mark[0].line = i - 1; ++c; break; }
     if( mark[0].line < 0 )
@@ -217,8 +239,12 @@ int Buffer::set_marks()
     if( mark[0].line < 0 && lines() > 22 ) { mark[0].line = 22; ++c; }
     }
   for( int i = 1; i < 10 && 100 * i <= lines(); ++i )
-    if( mark[i].line < 0 )
-      { mark[i].line = 100 * i - 1; ++c; }
+    {
+    if( mark[i].line >= 0 )
+      for( int j = i - 1; j >= 0; --j )
+        if( mark[i] == mark[j] ) { mark[i].line = -1; break; }
+    if( mark[i].line < 0 ) { mark[i].line = 100 * i - 1; ++c; }
+    }
   return c;
   }
 
@@ -317,33 +343,17 @@ long Buffer::character_offset( const Point & p, int & percent ) const
 
 // insert block b.[p1,p2) at p and move next
 bool Buffer::pputb( Point & p, const Basic_buffer & b, const Point & p1,
-                    const Point & p2, const int save_change )
+                    const Point & p2, const bool save_end )
   {
-  if( options.read_only || !pisvalid( p ) || !b.pisvalid( p1 ) ||
-      !b.pisvalid( p2 ) || p1 >= p2 ) return false;
+  if( options.read_only || !pisvalid( p ) ||
+      p1 >= p2 || !b.pisvalid( p1 ) || !b.pisvalid( p2 ) ) return false;
   const Point p0 = p;
   Basic_buffer::pputb( p, b, p1, p2 );
-  if( save_change )		// save_change == 0 only for undo/redo
-    {
-    const Point & after = ( save_change != 2 ) ? p0 : p;
-    Change_history::Record r( p0, after, modified_ );
-    r.add_atom( Change_history::Atom( p0, p ) );
-    change_history.add_record( *this, r );
-    modified_ = true;
-    }
-
-  const int line_dif = p2.line - p1.line;
-  std::vector< Point * > & pv = get_marks_and_pointers( *this );
-  for( unsigned i = 0; i < pv.size(); ++i )
-    if( *pv[i] > p0 && pv[i] != &p )
-      {
-      if( pv[i]->line == p0.line )
-        {
-        if( line_dif == 0 ) pv[i]->col += p2.col - p1.col;
-        else pv[i]->col += p2.col - p0.col;
-        }
-      pv[i]->line += line_dif;
-      }
+  Change_history::Record r( p0, save_end ? p : p0, modified_ );
+  r.add_atom( Change_history::Atom( p0, p ) );
+  change_history.add_record( *this, r );
+  modified_ = true;
+  adjust_marks_and_pointers( *this, p0, p0, p, &p );
   return true;
   }
 
@@ -363,7 +373,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
       if( pt.col > 0 && p.col >= pt.col )
         tmp.pputb( dummy, *this, bol( pt ), pt );
       }
-    const bool result = pputb( p, tmp, tmp.bof(), tmp.eof(), 2 );
+    const bool result = pputb( p, tmp, tmp.bof(), tmp.eof(), true );
     reset_appendable();
     return result;
     }
@@ -372,7 +382,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
     for( int i = 0; i < options.lmargin(); ++i ) tmp.pputc( dummy, ' ' );
     tmp.pputc( dummy, ch );
     reset_appendable();
-    return pputb( p, tmp, tmp.bof(), tmp.eof(), 2 );
+    return pputb( p, tmp, tmp.bof(), tmp.eof(), true );
     }
   else
     {
@@ -411,7 +421,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
           if( !options.auto_indent && options.lmargin() > 0 )
             {
             for( int i = 0; i < options.lmargin(); ++i ) tmp.pputc( dummy, ' ' );
-            force_append(); pputb( p2, tmp, tmp.bof(), tmp.eof(), 2 );
+            force_append(); pputb( p2, tmp, tmp.bof(), tmp.eof(), true );
             }
           }
         }
@@ -423,32 +433,16 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
 
 
 // delete block at [p1,p2)
-bool Buffer::pdelb( const Point & p1, const Point & p2, const int save_change )
+bool Buffer::pdelb( const Point & p1, const Point & p2, const bool save_end )
   {
   if( options.read_only || p1 >= p2 || !pisvalid( p1 ) || !pisvalid( p2 ) )
     return false;
-  if( save_change )		// save_change == 0 only for undo/redo
-    {
-    const Point & p0 = ( save_change != 2 ) ? p1 : p2;
-    Change_history::Record r( p0, p1, modified_ );
-    r.add_atom( Change_history::Atom( p1, p2, *this ) );
-    change_history.add_record( *this, r );
-    modified_ = true;
-    }
+  Change_history::Record r( save_end ? p2 : p1, p1, modified_ );
+  r.add_atom( Change_history::Atom( p1, p2, *this ) );
+  change_history.add_record( *this, r );
+  modified_ = true;
   Basic_buffer::pdelb( p1, p2 );
-
-  const int line_dif = p2.line - p1.line;
-  std::vector< Point * > & pv = get_marks_and_pointers( *this );
-  for( unsigned i = 0; i < pv.size(); ++i )
-    if( *pv[i] > p1 )
-      {
-      if( *pv[i] <= p2 ) *pv[i] = p1;
-      else
-        {
-        if( pv[i]->line == p2.line ) pv[i]->col += p1.col - p2.col;
-        pv[i]->line -= line_dif;
-        }
-      }
+  adjust_marks_and_pointers( *this, p1, p2, p1 );
   return true;
   }
 
@@ -457,8 +451,8 @@ bool Buffer::pdelb( const Point & p1, const Point & p2, const int save_change )
 bool Buffer::pdelc( Point & p, const bool back )
   {
   Point p1 = p;
-  if( back ) return ( pprev( p ) && pdelb( p, p1, 2 ) );
-  else return ( pnext( p1 ) && pdelb( p, p1 ) );
+  if( back ) return ( pprev( p ) && pdelb( p, p1, true ) );
+  return ( pnext( p1 ) && pdelb( p, p1 ) );
   }
 
 
@@ -477,18 +471,22 @@ bool Buffer::pdell( Point & p )
 bool Buffer::replace( const Point & p1, Point & p2, const Basic_buffer & b,
                       const Point & bp1, const Point & bp2 )
   {
-  Change_history::Record r( p1, p1, modified_ );
+  if( options.read_only || p1 > p2 || !pisvalid( p1 ) || !pisvalid( p2 ) ||
+      bp1 > bp2 || !b.pisvalid( bp1 ) || !b.pisvalid( bp2 ) ) return false;
+  if( p1 == p2 ) return pputb( p2, b, bp1, bp2 );
+  const Point p2_orig = p2;
+  if( bp1 == bp2 ) { p2 = p1; return pdelb( p1, p2_orig ); }
+  Change_history::Record r( p1, p1, modified_ );	// save begin
   Change_history::Atom a( p1, p2, *this );
-  if( !pdelb( p1, p2, 0 ) && a.bbp ) { delete a.bbp; a.bbp = 0; }
+  Basic_buffer::pdelb( p1, p2 );
   p2 = p1;
-  if( pputb( p2, b, bp1, bp2, 0 ) ) a.end = p2;
-  if( a.begin < a.end || a.bbp )
-    {
-    r.add_atom( a ); change_history.add_record( *this, r );
-    modified_ = true;
-    return true;
-    }
-  return false;
+  Basic_buffer::pputb( p2, b, bp1, bp2 );
+  a.end = p2;
+//  Change_history::Record r( p2_orig, p2, modified_ );	// save end
+  r.add_atom( a ); change_history.add_record( *this, r );
+  modified_ = true;
+  adjust_marks_and_pointers( *this, p1, p2_orig, p2, &p2 );
+  return true;
   }
 
 
@@ -499,19 +497,21 @@ bool Buffer::undo( Point & p )
   for( unsigned i = rec.atom_vector.size(); i > 0; )
     {
     Change_history::Atom & atom = rec.atom_vector[--i];
-    Basic_buffer *tmp = 0;
+    Basic_buffer * tmp = 0;
+    const Point end = atom.end;
     if( atom.begin < atom.end )
       {
       tmp = new Basic_buffer( *this, atom.begin, atom.end );
-      pdelb( atom.begin, atom.end, 0 );
+      Basic_buffer::pdelb( atom.begin, atom.end );
       }
     atom.end = atom.begin;
     if( atom.bbp )
       {
-      pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof(), 0 );
+      Basic_buffer::pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof() );
       delete atom.bbp;
       }
     atom.bbp = tmp;
+    adjust_marks_and_pointers( *this, atom.begin, end, atom.end, &atom.end );
     }
   p = rec.before;
   std::swap( modified_, rec.modified );
@@ -528,19 +528,21 @@ bool Buffer::redo( Point & p )
   for( unsigned i = 0; i < rec.atom_vector.size(); ++i )
     {
     Change_history::Atom & atom = rec.atom_vector[i];
-    Basic_buffer *tmp = 0;
+    Basic_buffer * tmp = 0;
+    const Point end = atom.end;
     if( atom.begin < atom.end )
       {
       tmp = new Basic_buffer( *this, atom.begin, atom.end );
-      pdelb( atom.begin, atom.end, 0 );
+      Basic_buffer::pdelb( atom.begin, atom.end );
       }
     atom.end = atom.begin;
     if( atom.bbp )
       {
-      pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof(), 0 );
+      Basic_buffer::pputb( atom.end, *atom.bbp, atom.bbp->bof(), atom.bbp->eof() );
       delete atom.bbp;
       }
     atom.bbp = tmp;
+    adjust_marks_and_pointers( *this, atom.begin, end, atom.end, &atom.end );
     }
   p = rec.after;
   std::swap( modified_, rec.modified );
