@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -89,10 +88,11 @@ public:
   bool pnext( Point & p ) const;
   bool pprev( Point & p ) const;
   bool pseek( Point & p, const int n ) const;
-  bool pvalid( Point & p ) const;
+  bool pvalid( Point & p ) const;	// make p valid, return old p status
 
   // insert block b.[p1,p2) at p and move next
-  bool pputb( Point & p, const Basic_buffer & b, const Point & p1, const Point & p2 );
+  bool pputb( Point & p, const Basic_buffer & b,
+              const Point & p1, const Point & p2 );
   // insert char and move next
   bool pputc( Point & p, const unsigned char ch );
   // change char and move next
@@ -141,7 +141,7 @@ class Buffer : public Basic_buffer
       {
       Point begin;	// Buffer position of delete or begin of insert
       Point end;		// Buffer position of end of insert
-      Basic_buffer *bbp;	// Basic_buffer containing deleted data
+      Basic_buffer * bbp;	// Basic_buffer containing deleted data
 
       // Creates an insert Atom ([p1,p2) -> inserted_text)
       Atom( const Point & p1, const Point & p2 )
@@ -211,7 +211,7 @@ public:
   Point mark[10];		// user bookmarks
   bool crlf;
 
-  explicit Buffer( const Options & opts, const std::string * const namep = 0 );
+  explicit Buffer( const std::string * const namep = 0 );
   int set_marks();		// set initial user bookmarks
   bool save( const std::string * namep = 0, bool set_name = true,
              bool append = false, Point p1 = Point(), Point p2 = Point() ) const;
@@ -225,12 +225,12 @@ public:
 
        // insert block b.[p1,p2) at p and move next
   bool pputb( Point & p, const Basic_buffer & b, const Point & p1,
-              const Point & p2, const int save_change = 1 );
+              const Point & p2, const bool save_end = false );
        // put char and move next
   bool pputc( Point & p, const unsigned char ch );
 
        // delete block at [p1,p2)
-  bool pdelb( const Point & p1, const Point & p2, const int save_change = 1 );
+  bool pdelb( const Point & p1, const Point & p2, const bool save_end = false );
        // delete char at 'p', or at '--p' if back is true
   bool pdelc( Point & p, const bool back = false );
   bool pdell( Point & p );	// delete line and set 'p' to bol()
