@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -72,7 +71,7 @@ void Bufhandle_vector::init()
   { first_ = 0; last_visited_ = 0; std::atexit( emergency_save ); }
 
 
-void Bufhandle_vector::save_all_named()
+const char * Bufhandle_vector::save_all_named()
   {
   int count = 0;
   bool done = true;
@@ -86,15 +85,16 @@ void Bufhandle_vector::save_all_named()
       catch( Buffer::Error ) { done = false; }
       }
     }
-  const char * const s = done ? "" : "Only ";
-  char buf[80];
-  snprintf( buf, sizeof buf, "%s%d named buffer(s) saved", s, count );
   Screen::show_status_lines();
-  Screen::show_message( buf );
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  snprintf( buf, bufsize, "%s%d named buffer(s) saved",
+            done ? "" : "Only ", count );
+  return buf;
   }
 
 
-void Bufhandle_vector::show_status()
+const char * Bufhandle_vector::show_status()
   {
   int mcount = 0, ucount = 0;
 
@@ -104,12 +104,13 @@ void Bufhandle_vector::show_status()
     if( data[i].buffer().name().empty() ) ++ucount;
     }
 
-  char buf[80];
-  snprintf( buf, sizeof buf,
+  Screen::show_status_lines();
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  snprintf( buf, bufsize,
             "**  %d modified and  %d unnamed out of  %d total buffer(s) **",
             mcount, ucount, handles() );
-  Screen::show_status_lines();
-  Screen::show_message( buf );
+  return buf;
   }
 
 
@@ -156,8 +157,7 @@ void Bufhandle_vector::set_first( const int ai )
   { first_ = ai % handles(); }
 
 
-int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
-                                          const std::string * const namep,
+int Bufhandle_vector::find_or_add_handle( const std::string * const namep,
                                           int i, const int line, const int col )
   {
   if( namep && namep->size() )
@@ -165,8 +165,8 @@ int Bufhandle_vector::find_or_add_handle( const Buffer::Options & opts,
       if( *namep == data[j].buffer().name() )
         return circular_index( j );
 
-  Buffer *bufp = new Buffer( opts, namep );
-  if( namep && namep->size() ) RC::apply_regex_options( *bufp );
+  Buffer * bufp = new Buffer( namep );
+  RC::apply_all_file_options( *bufp );
 
   if( i < 0 ) i = handles();
   else
@@ -186,7 +186,7 @@ void Bufhandle_vector::add_handle_if_pending_input_or_empty( const int line,
                                                              const int col )
   {
   if( !handles() || !isatty( fileno( stdin ) ) )
-    find_or_add_handle( RC::default_buffer_options(), 0, -1, line, col );
+    find_or_add_handle( 0, -1, line, col );
   }
 
 
