@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -42,8 +41,8 @@ public:
 namespace Bufhandle_vector {
 
 void init();
-void save_all_named();
-void show_status();
+const char * save_all_named();
+const char * show_status();
 
 int absolute_index( const int i );
 int circular_index( const int ai );
@@ -55,8 +54,7 @@ void next();
 void prev();
 void set_first( const int ai );
 
-int find_or_add_handle( const Buffer::Options & opts,
-                        const std::string * const namep = 0,
+int find_or_add_handle( const std::string * const namep = 0,
                         int i = -1, const int line = 0, const int col = 0 );
 void add_handle_if_pending_input_or_empty( const int line, const int col );
 void duplicate_handle( int i );
