@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -27,11 +26,15 @@ bool quoted_printable_decode( const std::string & in, std::string & out,
                               int * const idxp );
 
 unsigned char rot13( const unsigned char ch );
-
 unsigned char rot47( const unsigned char ch );
 
-bool utf8_encode( const std::string & in, std::string & out );
+bool ascii_encode( const std::string & in, std::string & out );
+
+bool utf8_encode( const std::string & in, std::string & out, const bool force );
 int utf8_to_ucs( const std::string & seq, const unsigned i, int * const lenp );
-int utf8_decode( const std::string & in, std::string & out, int * const idxp );
+int utf8_decode( const std::string & in, std::string & out, int * const idxp,
+                 const bool force );
+int remove_utf8_out_of_range( const std::string & in, std::string & out,
+                              int * const idxp );
 
 } // end namespace Encoding
