@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -24,17 +23,46 @@
 
 namespace ISO_8859 {
 
-enum {
+enum Code {
+     NBSP    = 0xA0,	// no-break space
+     IEXCLAM = 0xA1,	// inverted exclamation mark
+     CENT    = 0xA2,	// cent sign
+     POUND   = 0xA3,	// pound sign
+     EURO    = 0xA4,	// euro sign
+     YEN     = 0xA5,	// yen sign
      CSCARON = 0xA6,	// latin capital letter s with caron
+     SECTION = 0xA7,	// section sign
      SSCARON = 0xA8,	// latin small letter s with caron
+     COPY    = 0xA9,	// copyright sign
+     FEMIORD = 0xAA,	// feminine ordinal indicator
+     LDANGLE = 0xAB,	// left-pointing double angle quotation mark
+     NOT     = 0xAC,	// not sign
+     SHY     = 0xAD,	// soft hyphen
+     REG     = 0xAE,	// registered sign
+     MACRON  = 0xAF,	// macron
+     DEG     = 0xB0,	// degree sign
+     PLUSMIN = 0xB1,	// plus-minus sign
+     POW2    = 0xB2,	// superscript two
+     POW3    = 0xB3,	// superscript three
      CZCARON = 0xB4,	// latin capital letter z with caron
+     MICRO   = 0xB5,	// micro sign
+     PILCROW = 0xB6,	// pilcrow sign
+     MIDDOT  = 0xB7,	// middle dot
      SZCARON = 0xB8,	// latin small letter z with caron
+     POW1    = 0xB9,	// superscript one
+     MASCORD = 0xBA,	// masculine ordinal indicator
+     RDANGLE = 0xBB,	// right-pointing double angle quotation mark
+     CLIGOE  = 0xBC,	// latin capital ligature oe
+     SLIGOE  = 0xBD,	// latin small ligature oe
+     CYDIAER = 0xBE,	// latin capital letter y with diaeresis
+     IQUEST  = 0xBF,	// inverted question mark
      CAGRAVE = 0xC0,	// latin capital letter a with grave
      CAACUTE = 0xC1,	// latin capital letter a with acute
      CACIRCU = 0xC2,	// latin capital letter a with circumflex
      CATILDE = 0xC3,	// latin capital letter a with tilde
      CADIAER = 0xC4,	// latin capital letter a with diaeresis
      CARING  = 0xC5,	// latin capital letter a with ring above
+     CLIGAE  = 0xC6,	// latin capital ligature ae
      CCCEDI  = 0xC7,	// latin capital letter c with cedilla
      CEGRAVE = 0xC8,	// latin capital letter e with grave
      CEACUTE = 0xC9,	// latin capital letter e with acute
@@ -51,6 +79,7 @@ enum {
      COCIRCU = 0xD4,	// latin capital letter o with circumflex
      COTILDE = 0xD5,	// latin capital letter o with tilde
      CODIAER = 0xD6,	// latin capital letter o with diaeresis
+     MULT    = 0xD7,	// multiplication sign
      COSTROK = 0xD8,	// latin capital letter o with stroke
      CUGRAVE = 0xD9,	// latin capital letter u with grave
      CUACUTE = 0xDA,	// latin capital letter u with acute
@@ -58,12 +87,14 @@ enum {
      CUDIAER = 0xDC,	// latin capital letter u with diaeresis
      CYACUTE = 0xDD,	// latin capital letter y with acute
      CTHORN  = 0xDE,	// latin capital letter thorn (icelandic)
+     SSSHARP = 0xDF,	// latin small letter sharp s (german)
      SAGRAVE = 0xE0,	// latin small letter a with grave
      SAACUTE = 0xE1,	// latin small letter a with acute
      SACIRCU = 0xE2,	// latin small letter a with circumflex
      SATILDE = 0xE3,	// latin small letter a with tilde
      SADIAER = 0xE4,	// latin small letter a with diaeresis
      SARING  = 0xE5,	// latin small letter a with ring above
+     SLIGAE  = 0xE6,	// latin small ligature ae
      SCCEDI  = 0xE7,	// latin small letter c with cedilla
      SEGRAVE = 0xE8,	// latin small letter e with grave
      SEACUTE = 0xE9,	// latin small letter e with acute
@@ -80,6 +111,7 @@ enum {
      SOCIRCU = 0xF4,	// latin small letter o with circumflex
      SOTILDE = 0xF5,	// latin small letter o with tilde
      SODIAER = 0xF6,	// latin small letter o with diaeresis
+     DIV     = 0xF7,	// division sign
      SOSTROK = 0xF8,	// latin small letter o with stroke
      SUGRAVE = 0xF9,	// latin small letter u with grave
      SUACUTE = 0xFA,	// latin small letter u with acute
@@ -94,14 +126,14 @@ enum {
 
 
 // 'seq[i]' begins a escape sequence (the characters following a '\').
-// Returns the corresponding code and, in *lenp, the characters read.
+// Returns the corresponding code and, in '*lenp', the characters read.
 // Returns -1 if error.
 //
-int ISO_8859::escape( const std::string & seq, const unsigned i, int *lenp )
+int ISO_8859::escape( const std::string & seq, const unsigned i, int * lenp )
   {
   if( i >= seq.size() ) return -1;
   int len = 1;
-  unsigned ch = seq[i];
+  unsigned ch = (unsigned char)seq[i];
 
   switch( ch )
     {
@@ -121,9 +153,10 @@ int ISO_8859::escape( const std::string & seq, const unsigned i, int *lenp )
       if( ch > 255 ) return -1;
       break;
     case 'x':
-    case 'X': if( i + 2 >= seq.size() || ISO_8859::xtoi( seq[i+1] ) < 0 ||
-                  ISO_8859::xtoi( seq[i+2] ) < 0 ) return -1;
-              ch = (ISO_8859::xtoi( seq[i+1] ) << 4) + ISO_8859::xtoi( seq[i+2] );
+    case 'X': if( i + 2 >= seq.size() || ISO_8859::xvalue( seq[i+1] ) < 0 ||
+                  ISO_8859::xvalue( seq[i+2] ) < 0 ) return -1;
+              ch = ( ISO_8859::xvalue( seq[i+1] ) << 4 ) +
+                   ISO_8859::xvalue( seq[i+2] );
               len = 3;
               break;
     }
@@ -153,14 +186,14 @@ void ISO_8859::escapize( std::string & s )
         s.insert( i, 1, 'x' ); ++i;
         const unsigned char ch = s[i];
         s.insert( i, 1, ISO_8859::xdigit( ch >> 4 ) ); ++i;
-        s[i] = ISO_8859::xdigit( ch & 0x0f );
+        s[i] = ISO_8859::xdigit( ch & 0x0F );
         }
       }
     }
   }
 
 
-void ISO_8859::deescapize( std::string & s )
+void ISO_8859::deescapize_search_string( std::string & s )
   {
   for( unsigned i = 0; i + 1 < s.size(); ++i )
     {
@@ -182,12 +215,14 @@ bool ISO_8859::islower( const unsigned char ch )
     {
     case SSCARON:
     case SZCARON:
+    case SLIGOE:
     case SAGRAVE:
     case SAACUTE:
     case SACIRCU:
     case SATILDE:
     case SADIAER:
     case SARING :
+    case SLIGAE:
     case SCCEDI :
     case SEGRAVE:
     case SEACUTE:
@@ -224,12 +259,15 @@ bool ISO_8859::isupper( const unsigned char ch )
     {
     case CSCARON:
     case CZCARON:
+    case CLIGOE:
+    case CYDIAER:
     case CAGRAVE:
     case CAACUTE:
     case CACIRCU:
     case CATILDE:
     case CADIAER:
     case CARING :
+    case CLIGAE:
     case CCCEDI :
     case CEGRAVE:
     case CEACUTE:
@@ -270,23 +308,6 @@ bool ISO_8859::isalnum_( const unsigned char ch )
   }
 
 
-bool ISO_8859::isodigit( const unsigned char ch )  // is 'ch' an octal digit?
-  {
-  switch( ch )
-    {
-    case '0':
-    case '1':
-    case '2':
-    case '3':
-    case '4':
-    case '5':
-    case '6':
-    case '7': return true;
-    default: return false;
-    }
-  }
-
-
 const char * ISO_8859::control_name( const unsigned char ch )	// '^A' --> "^A"
   {
   static char buf[3];
@@ -322,12 +343,15 @@ unsigned char ISO_8859::tolower( const unsigned char ch )
     {
     case CSCARON: return SSCARON;
     case CZCARON: return SZCARON;
+    case CLIGOE : return SLIGOE;
+    case CYDIAER: return SYDIAER;
     case CAGRAVE: return SAGRAVE;
     case CAACUTE: return SAACUTE;
     case CACIRCU: return SACIRCU;
     case CATILDE: return SATILDE;
     case CADIAER: return SADIAER;
     case CARING : return SARING;
+    case CLIGAE : return SLIGAE;
     case CCCEDI : return SCCEDI;
     case CEGRAVE: return SEGRAVE;
     case CEACUTE: return SEACUTE;
@@ -363,12 +387,15 @@ unsigned char ISO_8859::toupper( const unsigned char ch )
     {
     case SSCARON: return CSCARON;
     case SZCARON: return CZCARON;
+    case SLIGOE : return CLIGOE;
+    case SYDIAER: return CYDIAER;
     case SAGRAVE: return CAGRAVE;
     case SAACUTE: return CAACUTE;
     case SACIRCU: return CACIRCU;
     case SATILDE: return CATILDE;
     case SADIAER: return CADIAER;
     case SARING : return CARING;
+    case SLIGAE : return CLIGAE;
     case SCCEDI : return CCCEDI;
     case SEGRAVE: return CEGRAVE;
     case SEACUTE: return CEACUTE;
@@ -397,6 +424,118 @@ unsigned char ISO_8859::toupper( const unsigned char ch )
   }
 
 
+void ISO_8859::map_to_ascii( const unsigned char ch, std::string & out )
+  {
+  if( ch < 128 ) out += ch;
+  else if( ch < 0xA0 ) { out += '\x1B'; out += ch - 0x40; }
+  else
+    {
+    const char * p = 0;
+    char c = 0;
+    switch( Code( ch ) )
+      {
+      case NBSP   : c = ' '; break;
+      case IEXCLAM: c = '!'; break;
+      case CENT   : c = 'c'; break;
+      case POUND  : c = 'L'; break;
+      case EURO   : p = "euro"; break;
+      case YEN    : c = 'Y'; break;
+      case CSCARON: c = 'S'; break;
+      case SECTION: p = "SS"; break;
+      case SSCARON: c = 's'; break;
+      case COPY   : p = "(C)"; break;
+      case FEMIORD: p = ".a"; break;
+      case LDANGLE: p = "<<"; break;
+      case NOT    : c = '!'; break;
+      case SHY    : c = '-'; break;
+      case REG    : c = 'R'; break;
+      case MACRON : c = '~'; break;
+      case DEG    : c = 'o'; break;
+      case PLUSMIN: p = "+-"; break;
+      case POW2   : p = "^2"; break;
+      case POW3   : p = "^3"; break;
+      case CZCARON: c = 'Z'; break;
+      case MICRO  : c = 'u'; break;
+      case PILCROW: c = 'P'; break;
+      case MIDDOT : c = '.'; break;
+      case SZCARON: c = 'z'; break;
+      case POW1   : p = "^1"; break;
+      case MASCORD: p = ".o"; break;
+      case RDANGLE: p = ">>"; break;
+      case CLIGOE : c = 'E'; break;
+      case SLIGOE : c = 'e'; break;
+      case CYDIAER: c = 'Y'; break;
+      case IQUEST : c = '?'; break;
+      case CAGRAVE:
+      case CAACUTE:
+      case CACIRCU:
+      case CATILDE:
+      case CADIAER:
+      case CARING : c = 'A'; break;
+      case CLIGAE : c = 'E'; break;
+      case CCCEDI : c = 'C'; break;
+      case CEGRAVE:
+      case CEACUTE:
+      case CECIRCU:
+      case CEDIAER: c = 'E'; break;
+      case CIGRAVE:
+      case CIACUTE:
+      case CICIRCU:
+      case CIDIAER: c = 'I'; break;
+      case CETH   : c = 'D'; break;
+      case CNTILDE: p = "N~"; break;
+      case COGRAVE:
+      case COACUTE:
+      case COCIRCU:
+      case COTILDE:
+      case CODIAER: c = 'O'; break;
+      case MULT   : c = 'x'; break;
+      case COSTROK: p = "Oe"; break;
+      case CUGRAVE:
+      case CUACUTE:
+      case CUCIRCU:
+      case CUDIAER: c = 'U'; break;
+      case CYACUTE: c = 'Y'; break;
+      case CTHORN : p = "Th"; break;
+      case SSSHARP: p = "ss"; break;
+      case SAGRAVE:
+      case SAACUTE:
+      case SACIRCU:
+      case SATILDE:
+      case SADIAER:
+      case SARING : c = 'a'; break;
+      case SLIGAE : c = 'e'; break;
+      case SCCEDI : c = 'c'; break;
+      case SEGRAVE:
+      case SEACUTE:
+      case SECIRCU:
+      case SEDIAER: c = 'e'; break;
+      case SIGRAVE:
+      case SIACUTE:
+      case SICIRCU:
+      case SIDIAER: c = 'i'; break;
+      case SETH   : c = 'd'; break;
+      case SNTILDE: p = "n~"; break;
+      case SOGRAVE:
+      case SOACUTE:
+      case SOCIRCU:
+      case SOTILDE:
+      case SODIAER: c = 'o'; break;
+      case DIV    : c = '/'; break;
+      case SOSTROK: p = "oe"; break;
+      case SUGRAVE:
+      case SUACUTE:
+      case SUCIRCU:
+      case SUDIAER: c = 'u'; break;
+      case SYACUTE: c = 'y'; break;
+      case STHORN : p = "th"; break;
+      case SYDIAER: c = 'y'; break;
+      }
+    if( p ) out += p; else out += c;
+    }
+  }
+
+
 unsigned char ISO_8859::xdigit( const int value )
   {
   if( value >= 0 && value <= 9 ) return '0' + value;
@@ -405,7 +544,7 @@ unsigned char ISO_8859::xdigit( const int value )
   }
 
 
-int ISO_8859::xtoi( const unsigned char ch )
+int ISO_8859::xvalue( const unsigned char ch )
   {
   switch( ch )
     {
