@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -42,16 +41,25 @@ namespace {
 const char * const     Program_name = "GNU Moe";
 const char * const     program_name = "moe";
 const char * const config_file_name = "moerc";
-const char * const     program_year = "2014";
+const char * const     program_year = "2015";
 const char * invocation_name = 0;
 
 
 void show_help()
   {
-  std::printf( "%s - A powerful and user-friendly text editor.\n", Program_name );
+  std::printf( "%s is a powerful, 8-bit clean, console text editor for ISO-8859 and\n"
+               "ASCII character encodings. It has a modeless, user-friendly interface,\n"
+               "online help, multiple windows, unlimited undo/redo capability, unlimited\n"
+               "line length, global search/replace (on all buffers at once), block\n"
+               "operations, automatic indentation, word wrapping, file name completion,\n"
+               "directory browser, duplicate removal from prompt histories, delimiter\n"
+               "matching, text conversion from/to UTF-8, romanization, etc.\n", Program_name );
   std::printf( "\nUsage: %s [global_options] [[+line[,col]] file_name [file_options]]...\n", invocation_name );
-  std::printf( "  'global_options' are a mix of editor options and default file_options.\n"
-               "  Most long option names have a negative form '--backup' '--no-backup'.\n"
+  std::printf( "\n'global_options' are a mix of editor options and default file_options.\n"
+               "Most long option names have a negative form '--backup' '--no-backup'.\n"
+               "\nEach file name may be preceded by '+line[,column]' to start the cursor\n"
+               "at the specified line and, optionally, column. 'line' and 'column' are\n"
+               "applied to subsequent files until new values are specified for them.\n"
                "\nEditor options:\n"
                "  -h, --help               display this help and exit\n"
                "  -V, --version            output version information and exit\n"
@@ -89,7 +97,7 @@ void show_version()
   {
   std::printf( "GNU %s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
+  std::printf( "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>\n"
                "This is free software: you are free to change and redistribute it.\n"
                "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
