@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -35,7 +34,7 @@ bool file_menu( const std::string & prompt, std::string & name );
 int get_filename( const std::string & prompt, std::string & name,
                   const int abort_key = 3 );
 
-bool is_regular( const std::string & name, const bool quiet = false );
+bool is_regular( const std::string & name );
 
 bool make_path( const std::string & name );
 
