@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -34,41 +33,48 @@
 #include "regex.h"
 
 
+// order of application of file options by place of specification:
+// 1 options specified in rc file as 'default file options'.
+// 2 options specified in rc file as 'file name dependent options'.
+// 3 options specified in command line as 'default file options'.
+// 4 options specified in command line as 'file options'.
+// (4 is not applied to files opened during the editing sesion).
+
 namespace RC {
 
 const Arg_parser * parserp;
 
-// priority of default_buffer_options_ by place of specification:
-// ( 0 = lowest, 3 = highest ).
-// 0 default priority.
-// 1 priority of option specified in rc file as 'default file option'.
-// 2 priority of option specified in rc file as 'file name dependent option'.
-// 3 priority of option specified in command line as 'default file option'.
-// 'file options' specified in command line are applied unconditionally.
-//
-struct Buffer_options_priorities
+class File_options
   {
-  int lmargin, rmargin;
-  int auto_indent, overwrite, read_only, word_wrap;
+  struct Record
+    {
+    int code;
+    int argument;
+    Record( const int c, const int arg ) : code( c ), argument( arg ) {}
+    };
+  std::vector< Record > data;
 
-  Buffer_options_priorities()
-    : lmargin( 0 ), rmargin( 0 ),
-    auto_indent( 0 ), overwrite( 0 ), read_only( 0 ), word_wrap( 0 ) {}
-
-  void reset() { *this = Buffer_options_priorities(); }
+public:
+  File_options() {}
+  void add_option( const int c, const int arg = 0 )
+    { data.push_back( Record( c, arg ) ); }
+  void reset() { data.clear(); }
+  int size() const { return data.size(); }
+  int code( const int i ) const { return data[i].code; }
+  int argument( const int i ) const { return data[i].argument; }
   };
 
 
 struct Regex_options
   {
   std::vector< std::string > regex_vector;
-  Buffer::Options buffer_options;
+  File_options file_options;
   };
 
 
 Options editor_options_;
-Buffer::Options default_buffer_options_;
-Buffer_options_priorities priorities;
+File_options default_file_options;
+File_options moerc_file_options;
 std::vector< Regex_options > regex_options_vector;
 
 
@@ -134,7 +140,7 @@ const char * optname( const int code )
   }
 
 
-bool is_buffer_option( const int code )
+bool is_editor_option( const int code )
   {
   switch( code )
     {
@@ -147,90 +153,70 @@ bool is_buffer_option( const int code )
     case read_only     :
     case no_read_only  :
     case word_wrap     :
-    case no_word_wrap  : return true;
+    case no_word_wrap  : return false;
     }
-  return false;
-  }
-
-
-bool is_editor_option( const int code )
-  { return ( !is_buffer_option( code ) ); }
-
-
-bool set_priority( int & old_priority, const int priority )
-  {
-  if( priority <= 0 ) return true;		// applied unconditionally
-  if( priority >= old_priority ) { old_priority = priority; return true; }
-  return false;
+  return true;
   }
 
 
 bool set_option_from_code( const int code, const std::string & arg,
-                           Options * const edop, Buffer::Options * const bufop,
-                           const int priority, const char * const msg )
+                           File_options & file_options,
+                           const char * const msg,
+                           const bool allow_editor_options = false )
   {
-  if( ( is_editor_option( code ) && !edop ) ||
-      ( is_buffer_option( code ) && !bufop ) )
+  if( !allow_editor_options && is_editor_option( code ) )
     { std::fprintf( stderr, "%serror: Option '%s' not allowed here.\n",
                     msg, optname( code ) ); return false; }
 
   int errcode = 0, n;
   switch( code )
     {
-    case auto_unmark:       edop->auto_unmark = true; break;
-    case no_auto_unmark:    edop->auto_unmark = false; break;
-    case backup:            edop->backup = true; break;
-    case no_backup:         edop->backup = false; break;
-//    case beep:              edop->beep = true; break;
-//    case no_beep:           edop->beep = false; break;
-    case exit_ask:          edop->exit_ask = true; break;
-    case no_exit_ask:       edop->exit_ask = false; break;
-    case ignore_case:       edop->ignore_case = true; break;
-    case no_ignore_case:    edop->ignore_case = false; break;
+    case auto_unmark:       editor_options_.auto_unmark = true; break;
+    case no_auto_unmark:    editor_options_.auto_unmark = false; break;
+    case backup:            editor_options_.backup = true; break;
+    case no_backup:         editor_options_.backup = false; break;
+//    case beep:              editor_options_.beep = true; break;
+//    case no_beep:           editor_options_.beep = false; break;
+    case exit_ask:          editor_options_.exit_ask = true; break;
+    case no_exit_ask:       editor_options_.exit_ask = false; break;
+    case ignore_case:       editor_options_.ignore_case = true; break;
+    case no_ignore_case:    editor_options_.ignore_case = false; break;
     case indent_step:
       if( !parse_int( arg, n ) ) errcode = 1;
-      else if( !edop->set_indent_step( n ) ) errcode = 2; break;
+      else if( !editor_options_.set_indent_step( n ) ) errcode = 2; break;
     case keep_lines:
       if( !parse_int( arg, n ) ) errcode = 1;
-      else if( !edop->set_keep_lines( n ) ) errcode = 2; break;
+      else if( !editor_options_.set_keep_lines( n ) ) errcode = 2; break;
     case max_windows:
       if( !parse_int( arg, n ) ) errcode = 1;
-      else if( !edop->set_max_windows( n ) ) errcode = 2; break;
-    case orphan_extra:      edop->orphan_extra = true; break;
-    case rectangle_mode:    edop->rectangle_mode = true; break;
-    case no_rectangle_mode: edop->rectangle_mode = false; break;
-    case smart_home:        edop->smart_home = true; break;
-    case no_smart_home:     edop->smart_home = false; break;
-    case search_wrap:       edop->search_wrap = true; break;
-    case no_search_wrap:    edop->search_wrap = false; break;
+      else if( !editor_options_.set_max_windows( n ) ) errcode = 2; break;
+    case orphan_extra:      editor_options_.orphan_extra = true; break;
+    case rectangle_mode:    editor_options_.rectangle_mode = true; break;
+    case no_rectangle_mode: editor_options_.rectangle_mode = false; break;
+    case smart_home:        editor_options_.smart_home = true; break;
+    case no_smart_home:     editor_options_.smart_home = false; break;
+    case search_wrap:       editor_options_.search_wrap = true; break;
+    case no_search_wrap:    editor_options_.search_wrap = false; break;
 
-    case auto_indent:    if( set_priority( priorities.auto_indent, priority ) )
-                           bufop->auto_indent = true; break;
-    case no_auto_indent: if( set_priority( priorities.auto_indent, priority ) )
-                           bufop->auto_indent = false; break;
+    case auto_indent:
+    case no_auto_indent:
+    case overwrite:
+    case no_overwrite:
+    case read_only:
+    case no_read_only:
+    case word_wrap:
+    case no_word_wrap:      file_options.add_option( code ); break;
     case lmargin:
       if( !parse_int( arg, n ) ) errcode = 1;
-      else if( set_priority( priorities.lmargin, priority ) &&
-               !bufop->set_lmargin( n - 1 ) ) errcode = 2;
+      else if( !Buffer::Options().set_lmargin( n - 1 ) ) errcode = 2;
+      else file_options.add_option( code, n - 1 ); break;
       break;
     case rmargin:
       if( !parse_int( arg, n ) ) errcode = 1;
-      else if( set_priority( priorities.rmargin, priority ) &&
-               !bufop->set_rmargin( n - 1 ) ) errcode = 2;
+      else if( !Buffer::Options().set_rmargin( n - 1 ) ) errcode = 2;
+      else file_options.add_option( code, n - 1 ); break;
       break;
-    case overwrite:      if( set_priority( priorities.overwrite, priority ) )
-                           bufop->overwrite = true; break;
-    case no_overwrite:   if( set_priority( priorities.overwrite, priority ) )
-                           bufop->overwrite = false; break;
-    case read_only:      if( set_priority( priorities.read_only, priority ) )
-                           bufop->read_only = true; break;
-    case no_read_only:   if( set_priority( priorities.read_only, priority ) )
-                           bufop->read_only = false; break;
-    case word_wrap:      if( set_priority( priorities.word_wrap, priority ) )
-                           bufop->word_wrap = true; break;
-    case no_word_wrap:   if( set_priority( priorities.word_wrap, priority ) )
-                           bufop->word_wrap = false; break;
-    default: std::fprintf( stderr, "%sinternal error: uncaught option\n", msg );
+    default: std::fprintf( stderr, "%sinternal error: uncaught option.\n", msg );
              return false;
     }
 
@@ -258,18 +244,33 @@ int parse_line_col( const std::string & s, int & line, int & col )
   return c;
   }
 
-} // end namespace RC
 
-
-RC::Options & RC::editor_options() { return editor_options_; }
-
-
-const Buffer::Options & RC::default_buffer_options()
-  { return default_buffer_options_; }
-
-
-void RC::apply_regex_options( Buffer & buffer )
+void apply_file_options( Buffer & buffer, const File_options & file_options )
   {
+  for( int i = 0; i < file_options.size(); ++i )
+    {
+    const int code = file_options.code( i );
+    const int arg = file_options.argument( i );
+    switch( code )
+      {
+      case auto_indent:    buffer.options.auto_indent = true; break;
+      case no_auto_indent: buffer.options.auto_indent = false; break;
+      case lmargin:        buffer.options.set_lmargin( arg ); break;
+      case rmargin:        buffer.options.set_rmargin( arg ); break;
+      case overwrite:      buffer.options.overwrite = true; break;
+      case no_overwrite:   buffer.options.overwrite = false; break;
+      case read_only:      buffer.options.read_only = true; break;
+      case no_read_only:   buffer.options.read_only = false; break;
+      case word_wrap:      buffer.options.word_wrap = true; break;
+      case no_word_wrap:   buffer.options.word_wrap = false; break;
+      }
+    }
+  }
+
+
+void apply_regex_options( Buffer & buffer )
+  {
+  if( buffer.name().empty() ) return;
   for( unsigned i = 0; i < regex_options_vector.size(); ++i )
     {
     const Regex_options & ro = regex_options_vector[i];
@@ -278,23 +279,24 @@ void RC::apply_regex_options( Buffer & buffer )
       if( Regex::match_filename( ro.regex_vector[j],
           Menu::my_basename( buffer.name() ) ) )
         break;
-    if( j >= ro.regex_vector.size() ) continue;
-    if( priorities.lmargin <= 2 )
-      buffer.options.set_lmargin( ro.buffer_options.lmargin() );
-    if( priorities.rmargin <= 2 )
-      buffer.options.set_rmargin( ro.buffer_options.rmargin() );
-    if( priorities.auto_indent <= 2 )
-      buffer.options.auto_indent = ro.buffer_options.auto_indent;
-    if( priorities.overwrite <= 2 )
-      buffer.options.overwrite = ro.buffer_options.overwrite;
-    if( priorities.read_only <= 2 )
-      buffer.options.read_only |= ro.buffer_options.read_only;
-    if( priorities.word_wrap <= 2 )
-      buffer.options.word_wrap = ro.buffer_options.word_wrap;
-    break;
+    if( j < ro.regex_vector.size() )
+      { apply_file_options( buffer, ro.file_options ); break; }
     }
   }
 
+} // end namespace RC
+
+
+RC::Options & RC::editor_options() { return editor_options_; }
+
+
+void RC::apply_all_file_options( Buffer & buffer )
+  {
+  apply_file_options( buffer, moerc_file_options );
+  apply_regex_options( buffer );
+  apply_file_options( buffer, default_file_options );
+  }
+
 
      // Returns true if first non whitespace character is '+' or '-'.
 bool RC::issigned( const std::string & s )
@@ -394,13 +396,13 @@ int RC::process_rcfile( const std::string & name )
           retval = 2; }
       else if( regex_status == 0 )
         { if( !set_option_from_code( parser.code( 0 ), parser.argument( 0 ),
-                                     &editor_options_, &default_buffer_options_,
-                                     1, msg ) ) retval = 2; }
+                                     moerc_file_options, msg, true ) )
+            retval = 2; }
       else
         {
-        if( !set_option_from_code( parser.code( 0 ), parser.argument( 0 ), 0,
-                                   &regex_options_vector.back().buffer_options,
-                                   0, msg ) ) retval = 2;
+        if( !set_option_from_code( parser.code( 0 ), parser.argument( 0 ),
+                                   regex_options_vector.back().file_options,
+                                   msg ) ) retval = 2;
         regex_status = 2;
         }
       }
@@ -423,8 +425,7 @@ int RC::process_options()
     const int code = parserp->code( argind );
     if( !code ) break;				// line number or file name
     if( !set_option_from_code( code, parserp->argument( argind ),
-                               &editor_options_,
-                               &default_buffer_options_, 3, "\n" ) )
+                               default_file_options, "\n", true ) )
       return 1;
     }
 
@@ -454,11 +455,10 @@ int RC::process_options()
     Buffer * bufferp = 0;
     std::string name( parserp->argument( argind++ ) );
     Menu::tilde_expansion( name );
-    if( Menu::is_regular( name, true ) )
+    if( Menu::is_regular( name ) )
       {
       try {
-        const int i = Bufhandle_vector::find_or_add_handle(
-                      default_buffer_options_, &name, -1, line, col );
+        const int i = Bufhandle_vector::find_or_add_handle( &name, -1, line, col );
         bufferp = &Bufhandle_vector::handle( i ).buffer();
         }
       catch( Buffer::Error e )
@@ -474,16 +474,17 @@ int RC::process_options()
       std::fprintf( stderr, "warning: '%s' is not a regular file\n", name.c_str() );
       non_regular_found = true;
       }
+    File_options file_options;
     for( ; argind < parserp->arguments(); ++argind )
       {						// Process file options
       const int code = parserp->code( argind );
       if( !code ) break;			// line number or file name
       if( bufferp &&
           !set_option_from_code( code, parserp->argument( argind ),
-                                 0, &bufferp->options, 0,
-                                 first_post ? "\n" : "" ) )
+                                 file_options, first_post ? "\n" : "" ) )
         return 1;
       }
+    if( bufferp ) apply_file_options( *bufferp, file_options );
     }
   // Add empty buffer if no files given or stdin is not a tty
   try { Bufhandle_vector::add_handle_if_pending_input_or_empty( line, col ); }
@@ -523,7 +524,7 @@ int RC::read_options( const int argc, const char * const argv[] )
 void RC::reset()
   {
   editor_options_.reset();
-  default_buffer_options_.reset();
-  priorities.reset();
+  default_file_options.reset();
+  moerc_file_options.reset();
   regex_options_vector.clear();
   }
