@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -29,28 +28,26 @@ namespace Regex {
 
 class Set
   {
+  std::vector< unsigned char > data;		// faster than bool
   int parsed_len_;
   bool in;
-  std::string data;
 
 public:
   explicit Set( const std::string & regex, const unsigned i0 = 0 );
   bool includes( const unsigned char ch ) const
-    {
-    return ( parsed_len_ && ( in == ( data.find( ch ) < data.size() ) ) );
-    }
+    { return ( parsed_len_ > 0 && in == data[ch] ); }
   int parsed_len() const { return parsed_len_; }
   };
 
 
 Set::Set( const std::string & regex, const unsigned i0 )
-  : parsed_len_( 0 ), in( true )
+  : data( 256, false ), parsed_len_( 0 ), in( true )
   {
   if( i0 + 2 >= regex.size() || regex[i0] != '[' ) return;
   unsigned i = i0 + 1;
   bool fail = true;
   if( regex[i] == '^' ) { ++i; in = false; }
-  if( regex[i] == ']' ) data += regex[i++];
+  if( regex[i] == ']' ) data[regex[i++]] = true;
 
   for( ; i < regex.size(); ++i )
     {
@@ -63,8 +60,8 @@ Set::Set( const std::string & regex, const unsigned i0 )
       ch1 = cht; i += len;
       }
     if( i + 2 >= regex.size() || regex[i+1] != '-' || regex[i+2] == ']' )
-      data += ch1;
-    else
+      data[ch1] = true;
+    else					// range
       {
       i += 2;
       unsigned char ch2 = regex[i];
@@ -74,22 +71,22 @@ Set::Set( const std::string & regex, const unsigned i0 )
         if( cht < 0 ) break;
         ch2 = cht; i += len;
         }
-      for( int c = ch1; c <= ch2; ++c ) data += c;
+      for( int c = ch1; c <= ch2; ++c ) data[c] = true;
       }
     }
   if( !fail ) parsed_len_ = i - i0;
   }
 
 
-bool match_regex( const Basic_buffer & buffer, Point & p,
-                  const std::string & regex, unsigned i,
-                  std::vector< std::string > & pieces, const bool icase );
+int match_regex( const Basic_buffer & buffer, Point & p,
+                 const std::string & regex, unsigned i,
+                 std::vector< std::string > & pieces, const bool icase );
 
-inline bool case_plus( const Basic_buffer & buffer, Point & p,
-                       const std::string & regex, unsigned & i,
-                       std::vector< std::string > & pieces, const bool icase )
+inline int case_plus( const Basic_buffer & buffer, Point & p,
+                      const std::string & regex, unsigned & i,
+                      std::vector< std::string > & pieces, const bool icase )
   {
-  if( ++i >= regex.size() ) return false;
+  if( ++i >= regex.size() ) return 2;
   const int ps = pieces.size();
   std::string piece;		// Find shortest matching sequence
   int counter = 0;		// Minimum number of matches required
@@ -100,68 +97,81 @@ inline bool case_plus( const Basic_buffer & buffer, Point & p,
   unsigned char ch1 = regex[i];
   if( ch1 == '\\' )
     {
-    if( i + 1 < regex.size() )
+    if( i + 1 >= regex.size() ) return 2;
+    const unsigned char chr = regex[i+1];
+    if( chr == '[' )
       {
-      const unsigned char chr = regex[i+1];
-      if( chr == '[' )
+      Set set( regex, i + 1 );		// Set of characters to match
+      if( !set.parsed_len() ) return 2;
+      i += set.parsed_len();
+      for( int j = 0; ; ++j )
         {
-        Set set( regex, i + 1 );		// Set of characters to match
-        if( !set.parsed_len() ) return false;
-        i += set.parsed_len();
-        for( int j = 0; j < counter || !match_regex( buffer, p, regex, i + 1, pieces, icase ); ++j )
+        if( j >= counter )
           {
-          const int ch2 = buffer.pgetc( p );
-          if( ch2 < 0 || !set.includes( ch2 ) ) return false;
-          piece += ch2;
+          const int tmp = match_regex( buffer, p, regex, i + 1, pieces, icase );
+          if( tmp == 0 ) break; if( tmp != 1 ) return 2;
           }
-        pieces.insert( pieces.begin() + ps, piece ); i = regex.size();
-        return true;
+        const int ch2 = buffer.pgetc( p );
+        if( ch2 < 0 || !set.includes( ch2 ) ) return 1;
+        piece += ch2;
         }
-      if( chr == 'w' || chr == 'W' )
+      pieces.insert( pieces.begin() + ps, piece ); i = regex.size();
+      return 0;
+      }
+    if( chr == 'w' || chr == 'W' )
+      {
+      ++i;
+      for( int j = 0; ; ++j )
         {
-        ++i;
-        for( int j = 0; j < counter || !match_regex( buffer, p, regex, i + 1, pieces, icase ); ++j )
+        if( j >= counter )
           {
-          const int ch2 = buffer.pgetc( p );
-          if( ch2 < 0 || ( chr == 'w' ) != ISO_8859::isalnum_( ch2 ) )
-            return false;
-          piece += ch2;
+          const int tmp = match_regex( buffer, p, regex, i + 1, pieces, icase );
+          if( tmp == 0 ) break; if( tmp != 1 ) return 2;
           }
-        pieces.insert( pieces.begin() + ps, piece ); i = regex.size();
-        return true;
+        const int ch2 = buffer.pgetc( p );
+        if( ch2 < 0 || ( chr == 'w' ) != ISO_8859::isalnum_( ch2 ) )
+          return 1;
+        piece += ch2;
         }
+      pieces.insert( pieces.begin() + ps, piece ); i = regex.size();
+      return 0;
       }
     int len, cht = ISO_8859::escape( regex, i + 1, &len );
-    if( cht < 0 ) return false;
+    if( cht < 0 ) return 2;
     ch1 = cht; i += len;
     }
-  for( int j = 0; j < counter || !match_regex( buffer, p, regex, i + 1, pieces, icase ); ++j )
+  for( int j = 0; ; ++j )
     {
+    if( j >= counter )
+      {
+      const int tmp = match_regex( buffer, p, regex, i + 1, pieces, icase );
+      if( tmp == 0 ) break; if( tmp != 1 ) return 2;
+      }
     const int ch2 = buffer.pgetc( p );
     if( ch2 < 0 ||
         ( icase && ISO_8859::tolower( ch1 ) != ISO_8859::tolower( ch2 ) ) ||
-        ( !icase && ch1 != ch2 ) ) return false;
+        ( !icase && ch1 != ch2 ) ) return 1;
     piece += ch2;
     }
   pieces.insert( pieces.begin() + ps, piece ); i = regex.size();
-  return true;
+  return 0;
   }
 
 
-// Returns true if substring of 'buffer' beginning at 'p' matches 'regex'.
+// Returns 0 if substring of 'buffer' beginning at 'p' matches 'regex'.
 // If found, sets 'p' after found substring and returns in 'pieces' the
 // text matching every \*, \? etc from the regular expresion.
 //
-bool match_regex( const Basic_buffer & buffer, Point & p,
-                  const std::string & regex, unsigned i,
-                  std::vector< std::string > & pieces, const bool icase )
+int match_regex( const Basic_buffer & buffer, Point & p,
+                 const std::string & regex, unsigned i,
+                 std::vector< std::string > & pieces, const bool icase )
   {
-  if( i >= regex.size() ) return true;
+  if( i >= regex.size() ) return 0;
   const Point old_p = p;
   const unsigned pieces_old_size = pieces.size();
-  bool fail = false;
+  int retval = 0;
 
-  for( ; i < regex.size() && !fail; ++i )
+  for( ; i < regex.size() && retval == 0; ++i )
     {
     unsigned char ch1 = regex[i];
     if( ch1 != '\\' )			// normal (not escaped) char
@@ -169,33 +179,35 @@ bool match_regex( const Basic_buffer & buffer, Point & p,
       const int ch2 = buffer.pgetc( p );
       if( ch2 < 0 || ( !icase && ch1 != ch2 ) ||
           ( icase && ISO_8859::tolower( ch1 ) != ISO_8859::tolower( ch2 ) ) )
-        fail = true;
+        retval = 1;
       continue;
       }
 
-    if( ++i >= regex.size() ) { fail = true; break; }
+    if( ++i >= regex.size() ) { retval = 2; break; }
     ch1 = regex[i];
     switch( ch1 )	// ch1 is first (maybe only) of a escape secuence
       {
-      case '^': if( p != buffer.bol( p ) || p == buffer.eof() ) fail = true; break;
-      case '$': if( p != buffer.eol( p ) ) fail = true; break;
-      case '<': if( !buffer.pisbow( p ) ) fail = true; break;
-      case '>': if( !buffer.piseow( p ) ) fail = true; break;
+      case '^': if( p != buffer.bol( p ) || p == buffer.eof() ) retval = 1; break;
+      case '$': if( p != buffer.eol( p ) ) retval = 1; break;
+      case '<': if( !buffer.pisbow( p ) ) retval = 1; break;
+      case '>': if( !buffer.piseow( p ) ) retval = 1; break;
       case '?': {
                 const int ch2 = buffer.pgetc( p );
-                if( ch2 < 0 ) { fail = true; break; }
+                if( ch2 < 0 ) { retval = 1; break; }
                 pieces.push_back( std::string( 1, ch2 ) );
                 } break;
       case '*': {			// Find shortest matching sequence
                 const int ps = pieces.size();
                 std::string piece;
-                while( !match_regex( buffer, p, regex, i + 1, pieces, icase ) )
+                while( true )
                   {
+                  const int tmp = match_regex( buffer, p, regex, i + 1, pieces, icase );
+                  if( tmp != 1 ) { retval = tmp; break; }
                   const int ch2 = buffer.pgetc( p );
-                  if( ch2 < 0 ) { fail = true; break; }
+                  if( ch2 < 0 ) { retval = 1; break; }
                   piece += ch2;
                   }
-                if( fail ) break;
+                if( retval ) break;
                 pieces.insert( pieces.begin() + ps, piece );
                 i = regex.size();
                 } break;
@@ -203,20 +215,22 @@ bool match_regex( const Basic_buffer & buffer, Point & p,
                 const Point begin = p;
                 Point end = p;
                 const int ps = pieces.size();
-                while( !match_regex( buffer, p, regex, i + 1, pieces, icase ) )
+                while( true )
                   {
+                  const int tmp = match_regex( buffer, p, regex, i + 1, pieces, icase );
+                  if( tmp != 1 ) { retval = tmp; break; }
                   const int ch2 = buffer.pgetc( p );
                   if( ch2 < 0 || ch2 == ')' || ch2 == ']' || ch2 == '}' )
-                    { fail = true; break; }
+                    { retval = 1; break; }
                   if( ch2 == '(' || ch2 == '[' || ch2 == '{' )
                     {
                     p = end;
                     if( buffer.set_to_matching_delimiter( p, true, true ) <= 0 ||
-                        !buffer.pnext( p ) ) { fail = true; break; }
+                        !buffer.pnext( p ) ) { retval = 1; break; }
                     }
                   end = p;
                   }
-                if( fail ) break;
+                if( retval ) break;
                 std::string piece;
                 buffer.to_string( begin, end, piece );
                 pieces.insert( pieces.begin() + ps, piece );
@@ -224,13 +238,13 @@ bool match_regex( const Basic_buffer & buffer, Point & p,
                 } break;
       case '[': {
                 Set set( regex, i );	// Set of characters to match
+                if( !set.parsed_len() ) { retval = 2; break; }
                 const int ch2 = buffer.pgetc( p );
-                if( ch2 < 0 || !set.includes( ch2 ) ) { fail = true; break; }
+                if( ch2 < 0 || !set.includes( ch2 ) ) { retval = 1; break; }
                 i += set.parsed_len() - 1;
                 pieces.push_back( std::string( 1, ch2 ) );
                 } break;
-      case '+': if( !case_plus( buffer, p, regex, i, pieces, icase ) )
-                  fail = true;
+      case '+': retval = case_plus( buffer, p, regex, i, pieces, icase );
                 break;
       case 'S': {			// Find longest matching sequence
                 std::string piece;
@@ -242,42 +256,45 @@ bool match_regex( const Basic_buffer & buffer, Point & p,
                   p = pn; piece += ch2;
                   }
                 if( piece.size() ) pieces.push_back( piece );
-                else fail = true;
+                else retval = 1;
                 } break;
       case 's': {			// Find shortest matching sequence
                 const int ps = pieces.size();
                 std::string piece;
-                while( !match_regex( buffer, p, regex, i + 1, pieces, icase ) )
+                while( true )
                   {
+                  const int tmp = match_regex( buffer, p, regex, i + 1, pieces, icase );
+                  if( tmp != 1 ) { retval = tmp; break; }
                   const int ch2 = buffer.pgetc( p );
-                  if( ch2 < 0 || !ISO_8859::isspace( ch2 ) ) { fail = true; break; }
+                  if( ch2 < 0 || !ISO_8859::isspace( ch2 ) ) { retval = 1; break; }
                   piece += ch2;
                   }
-                if( !fail ) { pieces.insert( pieces.begin() + ps, piece ); i = regex.size(); }
+                if( retval == 0 )
+                  { pieces.insert( pieces.begin() + ps, piece ); i = regex.size(); }
                 } break;
       case 'W':
       case 'w': {
                 const int ch2 = buffer.pgetc( p );
                 if( ch2 >= 0 && ( ch1 == 'w' ) == ISO_8859::isalnum_( ch2 ) )
                   pieces.push_back( std::string( 1, ch2 ) );
-                else fail = true;
+                else retval = 1;
                 } break;
       default: {
                int len, cht = ISO_8859::escape( regex, i, &len );
-               if( cht < 0 ) { fail = true; break; }
+               if( cht < 0 ) { retval = 2; break; }
                ch1 = cht;
                const int ch2 = buffer.pgetc( p );
                if( ch2 < 0 ||
                    ( icase && ISO_8859::tolower( ch1 ) != ISO_8859::tolower( ch2 ) ) ||
-                   ( !icase && ch1 != ch2 ) ) fail = true;
+                   ( !icase && ch1 != ch2 ) ) retval = 1;
                else i += len - 1;
                }
       }
     }
-  if( fail )
+  if( retval )
     { p = old_p;
     if( pieces_old_size != pieces.size() ) pieces.resize( pieces_old_size ); }
-  return !fail;
+  return ( retval );
   }
 
 } // end namespace Regex
@@ -287,12 +304,13 @@ bool match_regex( const Basic_buffer & buffer, Point & p,
 // If found, sets 'p1' at the first char of, and 'p2' after the
 // found substring and returns in 'pieces' the text matching every \*,
 // \? etc from the regular expresion.
+// Returns 0 if match, 1 if no match, 2 if trouble.
 //
-bool Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
-                  const std::string & regex, std::vector< std::string > & pieces,
-                  const bool icase, const bool backward )
+int Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
+                 const std::string & regex, std::vector< std::string > & pieces,
+                 const bool icase, const bool backward )
   {
-  if( regex.empty() ) return true;
+  if( regex.empty() ) return 0;
   if( pieces.size() ) pieces.clear();
 
   const int bs = regex.find('\\');
@@ -302,32 +320,36 @@ bool Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
     const bool same_size = ( header.size() == regex.size() );
     if( backward ) while( true )
       {
-      if( !buffer.rfind_text( p1, header, icase ) ) return false;
+      if( !buffer.rfind_text( p1, header, icase ) ) return 1;
       p2 = p1;
-      if( same_size ) { buffer.pseek( p2, header.size() ); return true; }
-      if( match_regex( buffer, p2, regex, 0, pieces, icase ) ) return true;
-      if( !buffer.pprev( p1 ) ) return false;
+      if( same_size ) { buffer.pseek( p2, header.size() ); return 0; }
+      const int tmp = match_regex( buffer, p2, regex, 0, pieces, icase );
+      if( tmp != 1 ) return tmp;
+      if( !buffer.pprev( p1 ) ) return 1;
       }
     else while( true )
       {
-      if( !buffer.find_text( p1, header, icase ) ) return false;
+      if( !buffer.find_text( p1, header, icase ) ) return 1;
       p2 = p1; buffer.pseek( p1, -header.size() );
-      if( same_size ) return true; else p2 = p1;
-      if( match_regex( buffer, p2, regex, 0, pieces, icase ) ) return true;
-      if( !buffer.pnext( p1 ) || p1 == buffer.eof() ) return false;
+      if( same_size ) return 0; else p2 = p1;
+      const int tmp = match_regex( buffer, p2, regex, 0, pieces, icase );
+      if( tmp != 1 ) return tmp;
+      if( !buffer.pnext( p1 ) || p1 == buffer.eof() ) return 1;
       }
     }
 
   if( backward ) while( true )
     {
-    if( !buffer.pprev( p1 ) ) return false; else p2 = p1;
-    if( match_regex( buffer, p2, regex, 0, pieces, icase ) ) return true;
+    if( !buffer.pprev( p1 ) ) return 1; else p2 = p1;
+    const int tmp = match_regex( buffer, p2, regex, 0, pieces, icase );
+    if( tmp != 1 ) return tmp;
     }
   else while( true )
     {
     p2 = p1;
-    if( match_regex( buffer, p2, regex, 0, pieces, icase ) ) return true;
-    else if( !buffer.pnext( p1 ) || p1 == buffer.eof() ) return false;
+    const int tmp = match_regex( buffer, p2, regex, 0, pieces, icase );
+    if( tmp != 1 ) return tmp;
+    if( !buffer.pnext( p1 ) || p1 == buffer.eof() ) return 1;
     }
   }
 
@@ -418,9 +440,10 @@ bool Regex::match_filename( const std::string & regex, const std::string & name,
                  {
                  int len, cht = ISO_8859::escape( regex, i1 + 1, &len );
                  if( cht < 0 ) { fail = true; break; }
-                 ch1 = cht; i1 += len - 1;
+                 ch1 = cht; i1 += len;
                  }
-               if( i2 >= name.size() || ch1 != name[i2++] ) fail = true;
+               if( i2 >= name.size() || ch1 != (unsigned char)name[i2++] )
+                 fail = true;
                break;
       }
     }
