@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -22,10 +21,11 @@ namespace Regex {
 // If found, sets 'p1' at the first char of, and 'p2' after the
 // found substring and returns in 'pieces' the text matching every \*,
 // \? etc from the regular expresion.
+// Returns 0 if match, 1 if no match, 2 if trouble.
 //
-bool find( const Basic_buffer & buffer, Point & p1, Point & p2,
-           const std::string & regex, std::vector< std::string > & pieces,
-           const bool icase = false, const bool backward = false );
+int find( const Basic_buffer & buffer, Point & p1, Point & p2,
+          const std::string & regex, std::vector< std::string > & pieces,
+          const bool icase = false, const bool backward = false );
 
 
 // Deletes block at [p1,p2) and replaces it with the text produced from
