@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -49,6 +48,18 @@ struct Lines_and_cursor
     : line( ln ), lines( lns ) {}
   };
 
+class Last_key
+  {
+  unsigned long prev, current;
+
+public:
+  Last_key() : prev( 0 ), current( -1UL ) {}
+  void set( const int key ) { prev = current; current = key; }
+  void unset() { prev = 0; current = -1UL; }
+  void shift( const int key ) { current <<= 16; current += ( key & 0xFFFF ); }
+  bool compare_prev() const { return ( prev == current ); }
+  } last_key;
+
 char * line_buf_;
 int line_buf_size;
 int height_, width_;
@@ -76,9 +87,17 @@ void initialize_ncurses()
   }
 
 
-void rep_addch( int n, const chtype ch )
+inline void rep_addch( int n, const chtype ch )	// only for printable chars
   { for( ; n > 0; --n ) waddch( stdscr, ch ); }
 
+void out_raw_char( chtype ch, const chtype atr )
+  {
+  if( ch < 32 ) ch = ( ch + '@' ) | A_UNDERLINE;
+  else if( ch >= 0x7F && ch < 0xA0 )
+    ch = ( ch + '@' - 0x80 ) | A_REVERSE | A_UNDERLINE;
+  waddch( stdscr, ch | atr );
+  }
+
 
 void out_raw_string( const int line, const int col, const std::string & s,
                      const int len, int pos = 0, const chtype atr = A_NORMAL )
@@ -88,7 +107,7 @@ void out_raw_string( const int line, const int col, const std::string & s,
   for( ; x < limit && pos < (int)s.size(); ++pos, ++x )
     {
     unsigned char ch = s[pos];
-    waddch( stdscr, atr | ch );
+    out_raw_char( ch, atr );
     }
   if( x < limit ) rep_addch( limit - x, atr | ' ' );
   }
@@ -116,7 +135,7 @@ void out_string( const int line, const int col, const std::string & s,
         case 'i': atr ^= A_REVERSE; ++pos; ch = 0; break;
         case 'u': atr ^= A_UNDERLINE; ++pos; ch = 0; break;
         }
-    if( ch ) { waddch( stdscr, atr | ch ); ++x; }
+    if( ch ) { out_raw_char( ch, atr ); ++x; }
     }
   if( x < limit ) rep_addch( limit - x, atr | ' ' );
   }
@@ -132,91 +151,104 @@ void remove_duplicates( std::vector< std::string > & history )
   }
 
 
-void alt_key_loop( int key )
+const char * alt_key_loop( int key )
   {
+  const char * msg = 0;
   Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
-  if( std::isdigit( key ) ) w.goto_mark( key - '0' );
+  last_key.shift( key );
+  if( std::isdigit( key ) ) msg = w.goto_mark( key - '0' );
   else switch( key )
     {
     case 'A': w.scroll_horizontal( -8 ); break;
-    case 'B': w.goto_begin_of_block(); break;
-    case 'C': w.goto_column(); break;
+    case 'B': msg = w.goto_begin_of_block(); break;
+    case 'C': msg = w.goto_column(); break;
     case 'D': w.center_cursor(); break;
-    case 'F': w.goto_matching_delimiter(); break;
-    case 'G': w.goto_matching_delimiter( false ); break;
-    case 'I': w.show_character_info(); break;
-    case 'K': w.goto_end_of_block(); break;
-    case 'L': w.goto_line(); break;
-    case 'O': w.goto_offset(); break;
+    case 'F': msg = w.goto_matching_delimiter(); break;
+    case 'G': msg = w.goto_matching_delimiter( false ); break;
+    case 'I': msg = w.show_character_info(); break;
+    case 'K': msg = w.goto_end_of_block(); break;
+    case 'L': msg = w.goto_line(); break;
+    case 'O': msg = w.goto_offset(); break;
     case 'S': w.scroll_horizontal( 8 ); break;
-    case 'T': w.show_utf8_code(); break;
+    case 'T': msg = w.show_utf8_code(); break;
     case 'U': w.goto_bof(); break;
     case 'V': w.goto_eof(); break;
     case 'W': w.move_page( false, true ); break;
     case 'Z': w.move_page( true, true ); break;
     }
+  return msg;
   }
 
 
-void control_k_loop( int key )
+const char * control_k_loop( int key )
   {
+  const char * msg = 0;
   Window & w = Window_vector::curwin();
   const Buffer * const old_bufferp = Block::bufferp();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
-  if( std::isdigit( key ) ) w.set_mark( key - '0' );
+  last_key.shift( key );
+  if( std::isdigit( key ) ) msg = w.set_mark( key - '0' );
   else switch( key )
     {
     case 'B': Block::set_begin( w.buffer(), w.pointer() );
               if( old_bufferp ) repaint( old_bufferp );
               break;
     case 'C': Window_vector::copy_block(); break;
-    case 'E': w.extend_marks(); break;
+    case 'E': msg = w.extend_marks(); break;
     case 'I': Window_vector::indent_block(); break;
     case 'K': Block::set_end( w.buffer(), w.pointer() );
               if( old_bufferp ) repaint( old_bufferp );
               break;
     case 'M': Window_vector::move_block(); break;
-    case 'R': Window_vector::read_block(); break;
-    case 'U': Window_vector::unindent_block(); break;
-    case 'W': Window_vector::write_block(); break;
+    case 'R': msg = Window_vector::read_block(); break;
+    case 'U': msg = Window_vector::unindent_block(); break;
+    case 'W': msg = Window_vector::write_block(); break;
     case 'Y': Window_vector::delete_block(); break;
     }
+  return msg;
   }
 
 
-void control_o_loop( int key, const char ** const msgp )
+const char * control_o_loop( int key )
   {
+  const char * msg = 0;
   Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
+  last_key.shift( key );
   switch( key )
     {
     case '1': Window_vector::encode_base64(); break;
-    case '2': Window_vector::decode_base64(); break;
+    case '2': msg = Window_vector::decode_base64(); break;
     case '3': Window_vector::encode_rot1347( true ); break;
     case '4': Window_vector::encode_rot1347( false ); break;
-    case '6': Window_vector::decode_quoted_printable_utf8( true ); break;
-    case '7': Window_vector::encode_utf8(); break;
-    case '8': Window_vector::decode_quoted_printable_utf8( false ); break;
+    case '5': Window_vector::encode_ascii_utf8( &msg, true ); break;
+    case '6': msg = Window_vector::decode_quoted_printable_utf8( true ); break;
+    case '7': if( Window_vector::encode_ascii_utf8( &msg, false,
+                    last_key.compare_prev() ) ) last_key.unset(); break;
+    case '8': msg = Window_vector::decode_quoted_printable_utf8( false,
+                      last_key.compare_prev() ); break;
     case 'C': Window_vector::center_line(); break;
-    case 'D': Window_vector::remove_duplicate_lines(); break;
-    case 'E': Window_vector::remove_duplicate_lines( true ); break;
-    case 'F': *msgp = Window_vector::search( 6, true, "", true ); break;
-    case 'L': Window_vector::change_case( false ); break;
-    case 'N': Window_vector::change_buffer_name(); break;
+    case 'D': msg = Window_vector::remove_duplicate_lines(); break;
+    case 'E': msg = Window_vector::remove_duplicate_lines( true ); break;
+    case 'F': msg = Window_vector::search( 6, true, "", true ); break;
+    case 'L': msg = Window_vector::change_case( false ); break;
+    case 'N': msg = Window_vector::change_buffer_name(); break;
+    case 'O': msg = Window_vector::remove_utf8_out_of_range(); break;
     case 'R': werase( stdscr ); wrefresh( stdscr ); repaint(); break;
     case 'S': Window_vector::split_window(); break;
-    case 'U': Window_vector::change_case( true ); break;
+    case 'U': msg = Window_vector::change_case( true ); break;
     }
+  return msg;
   }
 
 
@@ -227,9 +259,10 @@ int control_q_loop( int key, const char ** const msgp )
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
+  last_key.shift( key );
   switch( key )
     {
-    case KEY_F(2): Bufhandle_vector::save_all_named(); break;
+    case KEY_F(2): *msgp = Bufhandle_vector::save_all_named(); break;
     case 'C': if( Window_vector::close_and_exit( true ) == 0 ) return 0; break;
     case 'U': *msgp = Window_vector::copyright_update(); break;
     case 'X': if( Window_vector::close_and_exit( false ) == 0 ) return 0; break;
@@ -238,30 +271,33 @@ int control_q_loop( int key, const char ** const msgp )
   }
 
 
-void control_s_loop( int key, const std::string & version_msg )
+const char * control_s_loop( int key, const char * const version_msg )
   {
+  const char * msg = 0;
   Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
+  last_key.shift( key );
   switch( key )
     {
     case KEY_F(2):
     case KEY_F(4):
-    case KEY_F(8): w.show_multichar_value( key - KEY_F(0), true ); break;
+    case KEY_F(8): msg = w.show_multichar_value( key - KEY_F(0), true ); break;
     case '2':
     case '4':
-    case '8': w.show_multichar_value( key - '0', false ); break;
+    case '8': msg = w.show_multichar_value( key - '0', false ); break;
     case 'C': RC::editor_options().show_code = !RC::editor_options().show_code;
               show_status_lines(); break;
-    case 'G': Bufhandle_vector::show_status(); break;
-    case 'V': show_message( version_msg ); break;
+    case 'G': msg = Bufhandle_vector::show_status(); break;
+    case 'V': msg = version_msg; break;
     }
+  return msg;
   }
 
 
-void user_data( const std::string & s, const int base )
+const char * user_data( const std::string & s, const int base )
   {
   std::vector< int > buf;	// s is "big endian", buf is little endian
   int last = s.size() - 1;
@@ -271,19 +307,18 @@ void user_data( const std::string & s, const int base )
   if( base == 16 )
     for( int i = last; i >= 2; i -= 2 )
       {
-      const int lsd = ISO_8859::xtoi( s[i] );
-      const int msd = ( i > 2 ) ? ISO_8859::xtoi( s[i-1] ) : 0;
-      if( lsd < 0 || msd < 0 )
-        { show_message( "Invalid hexadecimal data" ); return; }
+      const int lsd = ISO_8859::xvalue( s[i] );
+      const int msd = ( i > 2 ) ? ISO_8859::xvalue( s[i-1] ) : 0;
+      if( lsd < 0 || msd < 0 ) return "Invalid hexadecimal data";
       buf.push_back( ( msd << 4 ) | lsd );
       }
   else
     for( int i = 0; i <= last; ++i )
       {
       if( base == 10 && !std::isdigit( s[i] ) )
-        { show_message( "Invalid decimal data" ); return; }
+        return "Invalid decimal data";
       if( base == 8 &&  !ISO_8859::isodigit( s[i] ) )
-        { show_message( "Invalid octal data" ); return; }
+        return "Invalid octal data";
       int carry = s[i] - '0';
       for( unsigned j = 0; j < buf.size(); ++j )
         {
@@ -300,10 +335,11 @@ void user_data( const std::string & s, const int base )
   else
     for( unsigned i = 0; i < buf.size(); ++i )
       Window_vector::add_char( buf[i], true );
+  return 0;
   }
 
 
-void user_control_char( const int abort_key )
+const char * user_control_char( const int abort_key )
   {
   static std::vector< std::string > history;
   const std::string prompt1( "(Ctrl)A-Z, Code(+) or Escape sequence (^C to abort): " );
@@ -315,7 +351,7 @@ void user_control_char( const int abort_key )
     key = show_message( prompt1, false, true );
     if( key == 8 || key == KEY_F(1) )
       { wait_kbhit(); Menu::help_menu( 8, KEY_F(1) ); }
-    else if( key == 3 || key == abort_key ) { wait_kbhit(); return; }	// ^C
+    else if( key == 3 || key == abort_key ) { wait_kbhit(); return 0; }	// ^C
     else if( key >= 256 || key == '\\' || std::isalnum( key ) ) break;
     else wait_kbhit();
     }
@@ -328,20 +364,21 @@ void user_control_char( const int abort_key )
     {
     history.push_back( std::string() );
     const int size = get_string( prompt2, history, abort_key );
-    if( size <= 0 ) return;
+    if( size <= 0 ) return 0;
     const std::string & s = history.back();
     if( size >= 2 && s[0] == '\\' )
       { int len; key = ISO_8859::escape( s, 1, &len );
         if( key >= 0 && len + 1 != size ) key = -1; }
     else if( s[0] >= '1' && s[0] <= '9' )
-      { user_data( s, 10 ); return; }
+      return user_data( s, 10 );
     else if( size >= 3 && s[0] == '0' && ( s[1] == 'x' || s[1] == 'X' ) )
-      { user_data( s, 16 ); return; }
-    else if( s[0] == '0' ) { user_data( s, 8 ); return; }
+      return user_data( s, 16 );
+    else if( s[0] == '0' ) return user_data( s, 8 );
     else key = -1;
     }
-  if( key < 0 || key >= 256 ) show_message( "Invalid code" );
-  else Window_vector::add_char( key, true );
+  if( key < 0 || key >= 256 ) return "Invalid code";
+  Window_vector::add_char( key, true );
+  return 0;
   }
 
 
@@ -381,6 +418,9 @@ int read_key( const bool wait, const bool get_key )
   return -1;
   }
 
+void discard_control_c()		// discards an already pressed ^C
+  { while( read_key( false, false ) == 3 ) read_key( false, true ); }
+
 } // end namespace Screen
 
 
@@ -520,11 +560,8 @@ int Screen::get_string( const std::string & prompt,
                  break;
       case '\n': {
                  std::string name( s, prompt.size(), s.size() );
-                 if( !usetab || name.empty() )
+                 if( !usetab || name.empty() || Menu::validate_filename( name ) )
                    { key = -2; history.back() = name; }
-                 else if( Menu::validate_filename( name ) )
-                   { if( !Menu::is_regular( name ) ) key = -1;
-                     else { key = -2; history.back() = name; } }
                  }
                  break;
       case 8        :						// ^H
@@ -635,20 +672,17 @@ void Screen::out_buffer_line( const Buffer & buffer, Point c, const int line )
 
   for( int i = 0; i < width_; ++i, ++c.col, ++p.col )
     {
-    int ch = buffer[p];
+    const int ch = buffer[p];
     if( ch < 0 || ( p == buffer.eol( p ) && ch == '\n' ) )
       { rep_addch( width_ - i, ' ' ); break; }
     int block_attr = Block::in_block( buffer, p ) ? A_REVERSE : 0;
     if( ch == '\t' )
       {
-      ch = ' ';
       while( i + 1 < width_ && c.col % 8 != 7 )
-        { waddch( stdscr, ch | block_attr ); ++i; ++c.col; }
+        { waddch( stdscr, block_attr | ' ' ); ++i; ++c.col; }
+      waddch( stdscr, block_attr | ' ' );
       }
-    else if( ch < 32 ) ch = ( ch + '@' ) | A_UNDERLINE;
-    else if( ch >= 127 && ch < 160 )
-      ch = ( ch + '@' - 128 ) | A_REVERSE | A_UNDERLINE;
-    waddch( stdscr, ch | block_attr );
+    else out_raw_char( ch, block_attr );
     }
   }
 
@@ -708,11 +742,9 @@ void Screen::show_feedback( const std::string & s )
   {
   const int line = height_ - 1;
 
-  save_lines_and_cursor( line, 1 );
   out_string( line, 0, s, width_ );
   wmove( stdscr, line, std::min( (int)s.size(), width_ - 1 ) );
   wrefresh( stdscr );
-  restore_lines_and_cursor();
   }
 
 
@@ -725,6 +757,7 @@ int Screen::show_message( const std::string & s, const bool get_key,
 
   out_string( line, 0, s, width_ );
   wmove( stdscr, line, std::min( (int)s.size(), width_ - 1 ) );
+  discard_control_c();
   int key;
   do key = wait_kbhit( take_cursor ? line : -1, get_key ); while( key < 0 );
 
@@ -765,7 +798,7 @@ int Screen::function_key( const int key )
   }
 
 
-int Screen::user_loop( const std::string & version_msg )
+int Screen::user_loop( const char * const version_msg )
   {
   show_message( version_msg );
   int retval = -1;
@@ -774,6 +807,7 @@ int Screen::user_loop( const std::string & version_msg )
     {
     const int key = wait_kbhit( -1, true, force_refresh );
     if( key < 0 ) continue;
+    last_key.set( key );
     const char * msg = 0;
     force_refresh = false;
     { Window & w = Window_vector::curwin();
@@ -781,28 +815,28 @@ int Screen::user_loop( const std::string & version_msg )
       {
       case  0: msg = Block::toggle_marking( w.buffer(), w.pointer() );
                break;						// ^SPACE
-      case  2: Window_vector::reformat(); break;		// ^B
+      case  2: msg = Window_vector::reformat(); break;		// ^B
       case  3: if( !Window_vector::close( true ) ) retval = 0; break;	// ^C
       case  6: msg = Window_vector::search( key ); break;	// ^F
       case  7: msg = Window_vector::search( key, true ); break;	// ^G
-      case 11: control_k_loop( key ); break;			// ^K
-      case 15: control_o_loop( key, &msg ); break;		// ^O
-      case 16: user_control_char( key ); break;			// ^P
+      case 11: msg = control_k_loop( key ); break;		// ^K
+      case 15: msg = control_o_loop( key ); break;		// ^O
+      case 16: msg = user_control_char( key ); break;		// ^P
       case 17: retval = control_q_loop( key, &msg ); break;	// ^Q
-      case 19: control_s_loop( key, version_msg ); break;	// ^S
+      case 19: msg = control_s_loop( key, version_msg ); break;	// ^S
       case 23: msg = Window_vector::search_word( key ); break;	// ^W
       case 24: if( !Window_vector::close( false ) ) retval = 0; break;	// ^X
       case 25: Window_vector::delete_line(); break;		// ^Y
-      case 27: alt_key_loop( key ); break;			// Alt or Esc
+      case 27: msg = alt_key_loop( key ); break;		// Alt or Esc
       case  8:							// ^H
       case KEY_F(1) : Menu::help_menu( 8, KEY_F(1) ); break;
-      case KEY_F(2) : Window_vector::save_file( key ); break;
-      case KEY_F(3) : Window_vector::load_file( key ); break;
+      case KEY_F(2) : msg = Window_vector::save_file( key ); break;
+      case KEY_F(3) : msg = Window_vector::load_file( key ); break;
       case KEY_F(4) : if( !Window_vector::load( -1 ) ) retval = 3; break;
       case KEY_F(5) : Window_vector::prev(); break;
       case KEY_F(6) : Window_vector::next(); break;
-      case KEY_F(7) : Window_vector::undo(); break;
-      case KEY_F(8) : Window_vector::redo(); break;
+      case KEY_F(7) : msg = Window_vector::undo(); break;
+      case KEY_F(8) : msg = Window_vector::redo(); break;
       case KEY_F(9) : Window_vector::copy_block(); break;
       case KEY_F(10): Menu::options_menu( w.buffer(), key ); break;
       case KEY_F(11): Window_vector::bufhandle_menu( key ); break;
@@ -821,7 +855,7 @@ int Screen::user_loop( const std::string & version_msg )
       case KEY_LEFT : w.goto_pprev(); break;
       case KEY_RIGHT: w.goto_pnext(); break;
       default       : if( key == '\n' ) force_refresh = true;
-                      Window_vector::add_char( key );
+                      msg = Window_vector::add_char( key );
       } }
     if( retval >= 0 ) break;
     { Window & w = Window_vector::curwin();
