@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -99,79 +98,78 @@ void Window::goto_eol()
   }
 
 
-void Window::goto_begin_of_block()
+const char * Window::goto_begin_of_block()
   {
   const Buffer & buffer = bhp->buffer();
   if( Block::bufferp() == &buffer && buffer.pisvalid( Block::begin() ) )
-    update_points( Block::begin(), true, true );
-  else if( !Block::bufferp() || Block::begin().line < 0 )
-    Screen::show_message( "Begin of block is not set" );
-  else Screen::show_message( "Begin of block is not in this file" );
+    { update_points( Block::begin(), true, true ); return 0; }
+  if( !Block::bufferp() || Block::begin().line < 0 )
+    return "Begin of block is not set";
+  return "Begin of block is not in this file";
   }
 
 
-void Window::goto_end_of_block()
+const char * Window::goto_end_of_block()
   {
   const Buffer & buffer = bhp->buffer();
   if( Block::bufferp() == &buffer && buffer.pisvalid( Block::end() ) )
-    update_points( Block::end(), true, true );
-  else if( !Block::bufferp() || Block::end().line < 0 )
-    Screen::show_message( "End of block is not set" );
-  else Screen::show_message( "End of block is not in this file" );
+    { update_points( Block::end(), true, true ); return 0; }
+  if( !Block::bufferp() || Block::end().line < 0 )
+    return "End of block is not set";
+  return "End of block is not in this file";
   }
 
 
-void Window::goto_line()
+const char * Window::goto_line()
   {
   static std::vector< std::string > history;
 
   history.push_back( std::string() );
   if( Screen::get_string( "Go to line (^C to abort): ", history ) <= 0 )
-    return;
+    return 0;
   const std::string & s = history.back();
   Point c = bhp->cursor;
   ++c.line;
   if( !RC::parse_relative_int( s, c.line ) || c.line <= 0 )
-    { Screen::show_message( "Invalid line number" ); history.pop_back();
-      return; }
+    { history.pop_back(); return "Invalid line number"; }
   --c.line;
   if( c.line > bhp->buffer().last_line() ) c.line = bhp->buffer().last_line();
   update_points( c, true, true, from_cursor );
+  return 0;
   }
 
 
-void Window::goto_column()
+const char * Window::goto_column()
   {
   static std::vector< std::string > history;
 
   history.push_back( std::string() );
   if( Screen::get_string( "Go to column (^C to abort): ", history ) <= 0 )
-    return;
+    return 0;
   const std::string & s = history.back();
   Point c = bhp->cursor;
   ++c.col;
   if( !RC::parse_relative_int( s, c.col ) || c.col <= 0 )
-    { Screen::show_message( "Invalid column number" ); history.pop_back();
-      return; }
+    { history.pop_back(); return "Invalid column number"; }
   --c.col;
   update_points( c, true, false, from_cursor );
+  return 0;
   }
 
 
-void Window::goto_offset()
+const char * Window::goto_offset()
   {
   static std::vector< std::string > history;
   const Buffer & buffer = bhp->buffer();
 
   history.push_back( std::string() );
   if( Screen::get_string( "Go to offset (^C to abort): ", history ) <= 0 )
-    return;
+    return 0;
   const std::string & s = history.back();
   const bool relative = RC::issigned( s );
   int offset = 0;
   if( !RC::parse_int( s, offset ) || ( !relative && offset < 0 ) )
-    { Screen::show_message( "Invalid offset value" ); history.pop_back();
-      return; }
+    { history.pop_back(); return "Invalid offset value"; }
   const int cr = buffer.crlf;
   Point p( 0, 0 );
   if( relative ) p = bhp->pointer;
@@ -197,10 +195,11 @@ void Window::goto_offset()
     if( !buffer.pisvalid( p ) ) p = buffer.bof();
     }
   update_points( p, true, true );
+  return 0;
   }
 
 
-void Window::goto_mark( const int i )
+const char * Window::goto_mark( const int i )
   {
   if( i >= 0 && i <= 9 )
     {
@@ -209,15 +208,17 @@ void Window::goto_mark( const int i )
       update_points( p, true, true );
     else
       {
-      char buf[32];
-      snprintf( buf, sizeof buf, "Mark %d not set", i );
-      Screen::show_message( buf );
+      int bufsize = 0;
+      char * const buf = Screen::line_buf( &bufsize );
+      snprintf( buf, bufsize, "Mark %d not set", i );
+      return buf;
       }
     }
+  return 0;
   }
 
 
-void Window::goto_matching_delimiter( const bool forward )
+const char * Window::goto_matching_delimiter( const bool forward )
   {
   Point p = bhp->pointer;
   const Buffer & buffer = bhp->buffer();
@@ -242,7 +243,8 @@ void Window::goto_matching_delimiter( const bool forward )
       res = buffer.set_to_matching_delimiter( p, forward );
     }
   if( res && p != bhp->pointer ) update_points( p );
-  if( res < 0 ) Screen::show_message( "No matching delimiter" );
+  if( res < 0 ) return "No matching delimiter";
+  return 0;
   }
 
 
@@ -326,28 +328,31 @@ void Window::scroll_vertical( const int lines )
   }
 
 
-void Window::extend_marks()
+const char * Window::extend_marks()
   {
   const int c = bhp->buffer().set_marks();
   if( c > 0 )
     {
-    char buf[32];
-    snprintf( buf, sizeof buf, "%d new mark(s) set", c );
-    Screen::show_message( buf );
+    int bufsize = 0;
+    char * const buf = Screen::line_buf( &bufsize );
+    snprintf( buf, bufsize, "%d new mark(s) set", c );
+    return buf;
     }
-  else Screen::show_message( "No new marks set" );
+  return "No new marks set";
   }
 
 
-void Window::set_mark( const int i )
+const char * Window::set_mark( const int i )
   {
   if( i >= 0 && i <= 9 )
     {
     bhp->buffer().mark[i] = bhp->pointer;
-    char buf[32];
-    snprintf( buf, sizeof buf, "Mark %d set", i );
-    Screen::show_message( buf );
+    int bufsize = 0;
+    char * const buf = Screen::line_buf( &bufsize );
+    snprintf( buf, bufsize, "Mark %d set", i );
+    return buf;
     }
+  return 0;
   }
 
 
@@ -360,11 +365,10 @@ void Window::repaint() const
   }
 
 
-void Window::show_character_info() const
+const char * Window::show_character_info() const
   {
   const Buffer & buffer = bhp->buffer();
   const Point p = bhp->pointer;
-  std::string s;
   int bufsize = 0;
   char * const buf = Screen::line_buf( &bufsize );
   const int ch = buffer[p];
@@ -376,20 +380,21 @@ void Window::show_character_info() const
     {
     int percent = 0;
     long offset = buffer.character_offset( p, percent );
-    snprintf( buf, bufsize, "Line %d  Pos %d  Offset %ld(0x%lX)  %d%%  ",
-              p.line + 1, p.col + 1, offset, offset, percent );
-    s = buf;
-    if( ch >= 0 )
-      snprintf( buf, bufsize, "Code %d(0x%02X/0%o)", ch, ch ,ch );
-    else
-      snprintf( buf, bufsize, "(End Of File)" );
+    const int len = snprintf( buf, bufsize, "Line %d  Pos %d  Offset %ld(0x%lX)  %d%%  ",
+                              p.line + 1, p.col + 1, offset, offset, percent );
+    if( len > 0 && len < bufsize )
+      {
+      if( ch >= 0 )
+        snprintf( buf + len, bufsize - len, "Code %d(0x%02X/0%o)", ch, ch ,ch );
+      else
+        snprintf( buf + len, bufsize - len, "(End Of File)" );
+      }
     }
-  s += buf;
-  Screen::show_message( s );
+  return buf;
   }
 
 
-void Window::show_multichar_value( const int size, const bool big_endian ) const
+const char * Window::show_multichar_value( const int size, const bool big_endian ) const
   {
   const Buffer & buffer = bhp->buffer();
   const Point p = bhp->pointer;
@@ -398,7 +403,7 @@ void Window::show_multichar_value( const int size, const bool big_endian ) const
   if( size < 2 || size > 8 )
     {
     snprintf( buf, bufsize, "*** Error: Bad Multichar Size: %d", size );
-    Screen::show_message( buf ); return;
+    return buf;
     }
   std::string text;
   const int sz = buffer.to_string( p, size, text );
@@ -424,11 +429,11 @@ void Window::show_multichar_value( const int size, const bool big_endian ) const
   else
     snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (End Of File)",
               offset, offset, percent );
-  Screen::show_message( buf );
+  return buf;
   }
 
 
-void Window::show_utf8_code()
+const char * Window::show_utf8_code()
   {
   const Buffer & buffer = bhp->buffer();
   Point p = bhp->pointer;
@@ -462,7 +467,7 @@ void Window::show_utf8_code()
   else
     snprintf( buf, bufsize, "*** Error: Bad Pointer: Line %d  Pos %d",
               p.line + 1, p.col + 1 );
-  Screen::show_message( buf );
+  return buf;
   }
 
 
@@ -472,9 +477,9 @@ void Window::show_status_line( const char * const prefix ) const
   {
   if( top_line_ < 0 ) return;
   const Buffer & buffer = bhp->buffer();
-  int size = 0;
-  char * const buf = Screen::line_buf( &size );
-  int offset = size - 41;
+  int bufsize = 0;
+  char * const buf = Screen::line_buf( &bufsize );
+  int offset = bufsize - 41;
   const int line = bhp->pcursor.line + 1, col = bhp->pcursor.col + 1;
   for( int i = line; i > 9999; i /= 10 ) --offset;
   for( int i = col; i > 999; i /= 10 ) --offset;
@@ -491,18 +496,18 @@ void Window::show_status_line( const char * const prefix ) const
                             RC::editor_options().rectangle_mode ? 'X' : ' ',
                             modified ? '*' : ' ', name,
                             modified ? " (Modified)" : "" );
-  if( len < offset ) std::memset( buf + len, ' ', offset - len );
-  snprintf( buf + offset, size - offset,
+  if( len > 0 && len < offset ) std::memset( buf + len, ' ', offset - len );
+  snprintf( buf + offset, bufsize - offset,
             "  Line %-4d Col %-3d %5s   F1 for help",
             line, col, Screen::clock_string() );
   if( RC::editor_options().show_code )
     {
-    offset = size - 13;
+    offset = bufsize - 13;
     const int ch = buffer[bhp->pointer];
     if( ch >= 0 )
-      snprintf( buf + offset, size - offset, "%3d(0x%02X)", ch, ch );
+      snprintf( buf + offset, bufsize - offset, "%3d(0x%02X)", ch, ch );
     else
-      snprintf( buf + offset, size - offset, "End of file" );
+      snprintf( buf + offset, bufsize - offset, "End of file" );
     }
   Screen::out_line( buf, top_line_, false, 'i' );
   }
