@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -41,13 +40,13 @@ public:
   void goto_eof();
   void goto_home();
   void goto_eol();
-  void goto_begin_of_block();
-  void goto_end_of_block();
-  void goto_line();
-  void goto_column();
-  void goto_offset();
-  void goto_mark( const int i );
-  void goto_matching_delimiter( const bool forward = true );
+  const char * goto_begin_of_block();
+  const char * goto_end_of_block();
+  const char * goto_line();
+  const char * goto_column();
+  const char * goto_offset();
+  const char * goto_mark( const int i );
+  const char * goto_matching_delimiter( const bool forward = true );
   void goto_pnext();
   void goto_pprev();
   void move_page( const bool down, const bool scroll = false );
@@ -55,13 +54,13 @@ public:
   void scroll_horizontal( const int cols );
   void scroll_vertical( const int lines );
 
-  void extend_marks();
-  void set_mark( const int i );
+  const char * extend_marks();
+  const char * set_mark( const int i );
 
   void repaint() const;
-  void show_character_info() const;
-  void show_multichar_value( const int size, const bool big_endian ) const;
-  void show_utf8_code();
+  const char * show_character_info() const;
+  const char * show_multichar_value( const int size, const bool big_endian ) const;
+  const char * show_utf8_code();
   void show_status_line( const char * const prefix = 0 ) const;
 
   enum From { from_top_left, from_cursor, from_pointer };
