@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -42,6 +41,23 @@ int curwin_;			// current window (the one with the cursor in)
 std::vector< Window > data;
 
 
+const char * already_being_edited( const std::string & name,
+                                   const Buffer & buffer )
+  {
+  if( name.size() )			// allow multiple unnamed buffers
+    for( int i = 0; i < Bufhandle_vector::handles(); ++i )
+      if( name == Bufhandle_vector::handle( i ).buffer().name() &&
+          &buffer != &Bufhandle_vector::handle( i ).buffer() )
+        {
+        int bufsize = 0;
+        char * const buf = Screen::line_buf( &bufsize );
+        snprintf( buf, bufsize, "File '%s' is already being edited", name.c_str() );
+        return buf;
+        }
+  return 0;
+  }
+
+
 bool no_block()
   {
   if( !Block::valid() ) { Screen::show_message( "No block" ); return true; }
@@ -87,6 +103,107 @@ void replace_and_repaint( Buffer & buffer, const Basic_buffer & tmp )
   Screen::repaint( &buffer );
   }
 
+
+bool insert_year( std::vector< int > & years, const int year )
+  {
+  if( year >= 1000 )
+    {
+    unsigned i = years.size();
+    while( i > 0 && years[i-1] > year ) --i;
+    if( i == 0 || years[i-1] < year )
+      { years.insert( years.begin() + i, year ); return true; }
+    }
+  return false;
+  }
+
+void print_year( Basic_buffer & outbuf, Point & p, int year )
+  {
+  while( year )
+    { outbuf.pputc( p, '0' + ( year % 10 ) ); outbuf.pprev( p ); year /= 10; }
+  p = outbuf.eof();
+  }
+
+bool insert_new_year( const Buffer & buffer, const Point & p0,
+                      const Point & pn, const Point & py,
+                      const Point & ph1, const Point & ph2,
+                      const int new_year, Basic_buffer & outbuf )
+  {
+  std::vector< int > years;	// years are ordered and duplicates removed
+  int year = 0, prev_year = 0;
+  for( Point p = py; p <= ph1; buffer.pnext( p ) )	// parse years
+    {
+    const int ch = buffer[p];
+    if( std::isdigit( ch ) ) { year *= 10; year += ch - '0'; continue; }
+    if( ch < 0 || ch == '\n' || ch == ',' || ch == '-' || p >= ph1 )
+      {
+      if( prev_year >= 1000 )
+        {
+        insert_year( years, prev_year );
+        for( int y = prev_year + 1; y < year; ++y ) insert_year( years, y );
+        }
+      insert_year( years, year );
+      if( ch == '-' ) prev_year = year; else prev_year = 0;
+      year = 0;
+      if( ch < 0 ) break;
+      }
+    else if( std::ispunct( ch ) ) year = 0;
+    }
+
+  bool change = insert_year( years, new_year );		// new_year inserted
+
+  const int rmargin = buffer.options.rmargin();
+  Point p = outbuf.eof();
+  int first_year = years[0];
+  prev_year = first_year;
+  for( unsigned i = 1; i <= years.size(); ++i )		// write years
+    {
+    year = ( i < years.size() ) ? years[i] : 0;
+    if( prev_year + 1 != year )				// end of range
+      {
+      const bool range = first_year + 1 < prev_year;	// 3 or more years
+      const int size = range ? 11 : 6;
+      if( first_year > years[0] ) outbuf.pputc( p, ',' );
+      if( outbuf.to_cursor( p ).col + size > rmargin + 1 )
+        {
+        outbuf.pputc( p, '\n' );
+        outbuf.pputb( p, buffer, p0, pn );	// add prefix of notice
+        }
+      else if( first_year > years[0] ) outbuf.pputc( p, ' ' );
+      if( first_year < prev_year )
+        {
+        print_year( outbuf, p, first_year );
+        if( range ) outbuf.pputc( p, '-' );
+        else { outbuf.pputc( p, ',' ); outbuf.pputc( p, ' ' ); }
+        }
+      print_year( outbuf, p, prev_year );
+      first_year = year;
+      }
+    prev_year = year;
+    }
+
+  if( ph1 < ph2 )				// write copyright holder
+    {
+    Point pt = p;
+    outbuf.pputb( p, buffer, ph1, ph2 );
+    if( outbuf.to_cursor( p ).col + 1 > rmargin + 1 )
+      {
+      outbuf.pputc( pt, '\n' );
+      outbuf.pputb( pt, buffer, p0, pn );	// add prefix of notice
+      }
+    else outbuf.pputc( pt, ' ' );
+    }
+
+  if( !change ) change = ( outbuf.last_line() != ( ph2.line - p0.line ) );
+  if( !change )
+    {
+    std::string s1, s2;
+    buffer.to_string( p0, ph2, s1 );
+    outbuf.to_string( outbuf.bof(), outbuf.eof(), s2 );
+    change = ( s1 != s2 );
+    }
+  return change;			// new_year inserted or list modified
+  }
+
 } // end namespace Window_vector
 
 
@@ -143,42 +260,39 @@ void Window_vector::bufhandle_menu( const int abort_key )
   }
 
 
-void Window_vector::change_buffer_name()
+const char * Window_vector::change_buffer_name()
   {
   Buffer & buffer = data[curwin_].buffer();
   std::string name( buffer.name() );
 
   if( Menu::get_filename( "New buffer name (^C to abort): ", name ) < 0 ||
       name == buffer.name() )
-    return;
-  if( name.size() )			// allow multiple unnamed buffers
-    for( int i = 0; i < Bufhandle_vector::handles(); ++i )
-      if( name == Bufhandle_vector::handle( i ).buffer().name() &&
-          &buffer != &Bufhandle_vector::handle( i ).buffer() )
-        {
-        std::string s( "File '" ); s += name; s += "' is already being edited";
-        Screen::show_message( s ); return;
-        }
-  buffer.name( name );
-  Screen::show_status_lines( &buffer );
+    return 0;
+  const char * const msg = already_being_edited( name, buffer );
+  if( !msg )
+    {
+    buffer.name( name );
+    Screen::show_status_lines( &buffer );
+    }
+  return msg;
   }
 
 
-void Window_vector::load_file( const int abort_key )
+const char * Window_vector::load_file( const int abort_key )
   {
   std::string name;
   if( Menu::get_filename( "Name of file to edit (^C to abort): ",
                           name, abort_key ) < 0 )
-    return;
+    return 0;
   const int old_handles = Bufhandle_vector::handles();
   int handle;
-  try { handle = Bufhandle_vector::find_or_add_handle(
-                 RC::default_buffer_options(), &name, curwin_ + 1 ); }
-  catch( Buffer::Error e ) { Screen::show_message( e.msg ); return; }
+  try { handle = Bufhandle_vector::find_or_add_handle( &name, curwin_ + 1 ); }
+  catch( Buffer::Error e ) { return e.msg; }
   curwin_ = handle;
   if( windows() > 1 && Bufhandle_vector::handles() > old_handles )
     load( windows() + 1 );
   else load();
+  return 0;
   }
 
 
@@ -189,23 +303,29 @@ void Window_vector::split_window()
   }
 
 
-void Window_vector::save_file( const int abort_key )
+const char * Window_vector::save_file( const int abort_key )
   {
   const Buffer & buffer = data[curwin_].buffer();
-
   std::string name( buffer.name() );
-  if( Menu::get_filename( "Name of file to save (^C to abort): ", name, abort_key ) <= 0 )
-    return;
+
+  if( Menu::get_filename( "Name of file to save (^C to abort): ",
+      name, abort_key ) <= 0 )
+    return 0;
+  const char * const msg = already_being_edited( name, buffer );
+  if( msg ) return msg;
   bool done;
   try { done = buffer.save( &name ); }
-  catch( Buffer::Error e ) { Screen::show_message( e.msg ); return; }
+  catch( Buffer::Error e ) { return e.msg; }
   if( done )
     {
     Screen::show_status_lines( &buffer );
-    std::string s( "File '" ); s += name; s += "' saved";
-    Screen::show_message( s );
+    int bufsize = 0;
+    char * const buf = Screen::line_buf( &bufsize );
+    snprintf( buf, bufsize, "File '%s' saved", name.c_str() );
+    return buf;
     }
-  else if( !read_only( buffer ) ) Screen::show_message( "File not saved" );
+  if( !read_only( buffer ) ) return "File not saved";
+  return 0;
   }
 
 
@@ -310,14 +430,19 @@ Window & Window_vector::window( int i )
 int Window_vector::windows() { return data.size(); }
 
 
-void Window_vector::add_char( const int key, const bool force )
+const char * Window_vector::add_char( const int key, const bool force )
   {
   Window & w = data[curwin_];
   if( w.buffer().options.read_only )
     {
-    if( !force && key == ' ' ) w.move_vertical( w.height() - 2 );
-    else read_only( w.buffer() );
-    return;
+    if( !force ) switch( key )
+      {
+      case ' ': w.move_vertical( w.height() - 2 ); return 0;
+      case '/': return search( 6 );				// ^F
+      case 'n': return search( 7, true );			// ^G
+      }
+    read_only( w.buffer() );
+    return 0;
     }
 
   if( key >= 0 && key < 256 &&
@@ -326,10 +451,12 @@ void Window_vector::add_char( const int key, const bool force )
         Screen::repaint( &w.buffer() ); }
   else
     {
-    char buf[64];
-    snprintf( buf, sizeof buf, "Key code %d(0x%02X) ignored", key, key );
-    Screen::show_message( buf );
+    int bufsize = 0;
+    char * const buf = Screen::line_buf( &bufsize );
+    snprintf( buf, bufsize, "Key code %d(0x%02X) ignored", key, key );
+    return buf;
     }
+  return 0;
   }
 
 
@@ -396,38 +523,38 @@ void Window_vector::move_block()
   }
 
 
-void Window_vector::read_block()
+const char * Window_vector::read_block()
   {
   Window & w = data[curwin_];
-  if( read_only( w.buffer() ) ) return;
+  if( read_only( w.buffer() ) ) return 0;
 
   std::string name;
   if( Menu::get_filename( "Name of file to insert (^C to abort): ", name ) <= 0 )
-    return;
+    return 0;
   try {
-    Buffer tmp( RC::default_buffer_options(), &name );
+    Buffer tmp( &name );
     if( tmp.empty() ) throw Buffer::Error( "File not found" );
     Point p = w.pointer();
     if( w.buffer().pputb( p, tmp, tmp.bof(), tmp.eof() ) )
       Screen::repaint( &w.buffer() );
     }
-  catch( Buffer::Error e ) { Screen::show_message( e.msg ); }
+  catch( Buffer::Error e ) { return e.msg; }
+  return 0;
   }
 
 
-void Window_vector::write_block()
+const char * Window_vector::write_block()
   {
-  if( no_block() ) return;
+  if( no_block() ) return 0;
   std::string name;
   if( Menu::get_filename( "Name of file to write (^C to abort): ", name ) <= 0 )
-    return;
+    return 0;
   try {
     if( !RC::editor_options().rectangle_mode )
       Block::bufferp()->save( &name, false, false, Block::begin(), Block::end() );
     else
       {
-      Buffer::Options options;
-      Buffer tmp( options );
+      Buffer tmp;
       Block::save_block_position();
       Block::copy_block( tmp, tmp.bof() );
       Block::restore_block_position();
@@ -439,7 +566,8 @@ void Window_vector::write_block()
       Block::reset(); Screen::repaint( bufferp );
       }
     }
-  catch( Buffer::Error e ) { Screen::show_message( e.msg ); }
+  catch( Buffer::Error e ) { return e.msg; }
+  return 0;
   }
 
 
@@ -472,12 +600,12 @@ void Window_vector::indent_block()
   }
 
 
-void Window_vector::unindent_block()
+const char * Window_vector::unindent_block()
   {
   Buffer & buffer = data[curwin_].buffer();
 
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
-      read_only( buffer ) ) return;
+      read_only( buffer ) ) return 0;
   Point p = Block::begin();
   bool fail = false;
   while( buffer.bol( p ) < Block::end() && !fail )
@@ -487,7 +615,7 @@ void Window_vector::unindent_block()
       fail = true;
     ++p.line;
     }
-  if( fail ) { Screen::show_message( "Can't unindent more" ); return; }
+  if( fail ) return "Can't unindent more";
 
   bool first_post = true;
   const bool ow = buffer.options.overwrite;
@@ -521,30 +649,33 @@ void Window_vector::unindent_block()
   buffer.options.overwrite = ow;
   if( RC::editor_options().auto_unmark ) Block::reset();
   Screen::repaint( &buffer );
+  return 0;
   }
 
 
-void Window_vector::undo()
+const char * Window_vector::undo()
   {
   Window & w = data[curwin_];
 
   if( !read_only( w.buffer() ) )
     {
     if( w.buffer().undo( w.pointer() ) ) Screen::repaint( &w.buffer() );
-    else Screen::show_message( "Nothing to undo" );
+    else return "Nothing to undo";
     }
+  return 0;
   }
 
 
-void Window_vector::redo()
+const char * Window_vector::redo()
   {
   Window & w = data[curwin_];
 
   if( !read_only( w.buffer() ) )
     {
     if( w.buffer().redo( w.pointer() ) ) Screen::repaint( &w.buffer() );
-    else Screen::show_message( "Nothing to redo" );
+    else return "Nothing to redo";
     }
+  return 0;
   }
 
 
@@ -576,21 +707,22 @@ void Window_vector::center_line()
   }
 
 
-void Window_vector::reformat()
+const char * Window_vector::reformat()
   {
   Window & w = data[curwin_];
   Buffer_handle & bh = w.buffer_handle();
   Buffer & buffer = bh.buffer();
-  const Point bop = buffer.bop( bh.pointer );
-  const Point eop = buffer.eop( bh.pointer );
-  const Point bot = buffer.bot( bop );
+  const Point bop = buffer.bop( bh.pointer );	// begin of paragraph
+  const Point eop = buffer.eop( bh.pointer );	// end of paragraph
+  const Point bot = buffer.bot( bop );		// begin of text in line
 
   if( !read_only( buffer ) && bop < eop )
     {
+    const int rmargin = buffer.options.rmargin();
     const int lmargin = std::max( buffer.options.lmargin(),
-                                  std::min( buffer.options.rmargin() - 1,
-                                            bh.cursor.col ) );
-    Buffer tmp( buffer.options );
+                                  std::min( rmargin - 1, bh.cursor.col ) );
+    Buffer tmp;
+    tmp.options.set_rmargin( rmargin );
     tmp.options.auto_indent = false;
     tmp.options.overwrite = false;
     tmp.options.word_wrap = true;
@@ -602,11 +734,11 @@ void Window_vector::reformat()
     for( Point p = bot; p < eop; )
       {
       unsigned char ch = buffer.pgetc( p );
-      if( !ISO_8859::isspace( ch ) )
+      if( !std::isspace( ch ) )			// exclude no-break space
         { tmp.pputc( dummy, ch ); dummy = tmp.eof(); space = false; }
       else if( !space ) { tmp.pputc( dummy, ' ' ); space = true; }
       if( Screen::kbhit() >= 0 && Screen::wait_kbhit() == 3 )
-        { Screen::show_message( "Aborted" ); return; }
+        return "Aborted";
       }
     if( space ) tmp.pdelc( dummy, true );
     if( dummy != tmp.bol( dummy ) ) tmp.pputc( dummy, '\n' );
@@ -627,14 +759,15 @@ void Window_vector::reformat()
       w.update_points( p, true, false, Window::from_cursor );
       }
     }
+  return 0;
   }
 
 
-void Window_vector::change_case( const bool to_caps )
+const char * Window_vector::change_case( const bool to_caps )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || read_only( buffer ) )
-    return;
+    return 0;
   Basic_buffer tmp;
   Point p = tmp.bof();
   bool change = false;
@@ -656,8 +789,9 @@ void Window_vector::change_case( const bool to_caps )
         { ch = ISO_8859::tolower( ch ); change = true; }
       tmp.pputc( p, ch );
       }
-  if( change ) replace_and_repaint( buffer, tmp );
-  else Screen::show_message( "No letters to change in block" );
+  if( !change ) return "No letters to change in block";
+  replace_and_repaint( buffer, tmp );
+  return 0;
   }
 
 
@@ -682,11 +816,11 @@ void Window_vector::encode_base64()
   }
 
 
-void Window_vector::decode_base64()
+const char * Window_vector::decode_base64()
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
-      read_only( buffer ) ) return;
+      read_only( buffer ) ) return 0;
   std::string in, out;
   Block::bufferp()->to_string( Block::begin(), Block::end(), in );
   for( int i = in.size() - 1; i >= 0; --i )
@@ -701,39 +835,45 @@ void Window_vector::decode_base64()
     while( ISO_8859::isspace( buffer[p] ) ) buffer.pnext( p );
     if( idx == 0 && buffer.pisvalid( p ) )
       data[curwin_].update_points( p, true, true );
-    Screen::show_message( "Invalid Base64 data in block" );
-    return;
+    return "Invalid Base64 data in block";
     }
   Basic_buffer tmp;
   Point p = tmp.bof();
   for( unsigned i = 0; i < out.size(); ++i )
     tmp.pputc( p, out[i] );
   replace_and_repaint( buffer, tmp );
+  return 0;
   }
 
 
-void Window_vector::encode_utf8()
+bool Window_vector::encode_ascii_utf8( const char ** const msgp,
+                                       const bool mode_a, const bool force )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
-      read_only( buffer ) ) return;
+      read_only( buffer ) ) { *msgp = 0; return false; }
   std::string in, out;
   Block::bufferp()->to_string( Block::begin(), Block::end(), in );
-  if( !Encoding::utf8_encode( in, out ) )
-    { Screen::show_message( "Text in block is already UTF-8 encoded" ); return; }
+  if( mode_a && !Encoding::ascii_encode( in, out ) )
+    { *msgp = "Text in block is already ASCII encoded"; return false; }
+  if( !mode_a && !Encoding::utf8_encode( in, out, force ) )
+    { *msgp = "Text in block is already UTF-8 encoded"; return false; }
   Basic_buffer tmp;
   Point p = tmp.bof();
   for( unsigned i = 0; i < out.size(); ++i )
     tmp.pputc( p, out[i] );
   replace_and_repaint( buffer, tmp );
+  *msgp = "Done";
+  return true;
   }
 
 
-void Window_vector::decode_quoted_printable_utf8( const bool mode_q )
+const char * Window_vector::decode_quoted_printable_utf8( const bool mode_q,
+                                                          const bool force )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
-      read_only( buffer ) ) return;
+      read_only( buffer ) ) return 0;
   std::string in, out;
   Block::bufferp()->to_string( Block::begin(), Block::end(), in );
   if( mode_q )
@@ -745,14 +885,13 @@ void Window_vector::decode_quoted_printable_utf8( const bool mode_q )
       Point p = Block::begin();
       if( buffer.pisvalid( p ) && buffer.pseek( p, idx ) )
         data[curwin_].update_points( p, true, true );
-      Screen::show_message( "Invalid Quoted-Printable data in block" );
-      return;
+      return "Invalid Quoted-Printable data in block";
       }
     }
   else
     {
     int idx;
-    const int tmp = Encoding::utf8_decode( in, out, &idx );
+    const int tmp = Encoding::utf8_decode( in, out, &idx, force );
     if( tmp != 0 )
       {
       Block::disable_marking();
@@ -761,22 +900,49 @@ void Window_vector::decode_quoted_printable_utf8( const bool mode_q )
         data[curwin_].update_points( p, true, true );
       if( tmp > 0 )
         {
-        char buf[64];
-        snprintf( buf, sizeof buf, "UTF-8 code 'U%04X' out of range in block", tmp );
-        Screen::show_message( buf );
+        int bufsize = 0;
+        char * const buf = Screen::line_buf( &bufsize );
+        snprintf( buf, bufsize, "UTF-8 code 'U%04X' out of range in block", tmp );
+        return buf;
         }
       else
-        Screen::show_message( "Invalid UTF-8 data in block" );
-      return;
+        return "Invalid UTF-8 data in block";
       }
     }
-  if( in == out )
-    { Screen::show_message( "Text in block is already decoded" ); return; }
+  if( in == out ) return "Text in block is already decoded";
   Basic_buffer tmp;
   Point p = tmp.bof();
   for( unsigned i = 0; i < out.size(); ++i )
     tmp.pputc( p, out[i] );
   replace_and_repaint( buffer, tmp );
+  return "Done";
+  }
+
+
+const char * Window_vector::remove_utf8_out_of_range()
+  {
+  Buffer & buffer = data[curwin_].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return 0;
+  std::string in, out;
+  Block::bufferp()->to_string( Block::begin(), Block::end(), in );
+  int idx;
+  const int ret = Encoding::remove_utf8_out_of_range( in, out, &idx );
+  if( ret == 0 ) return "Text in block is already decoded";
+  if( ret < 0 )
+    {
+    Block::disable_marking();
+    Point p = Block::begin();
+    if( buffer.pisvalid( p ) && buffer.pseek( p, idx ) )
+      data[curwin_].update_points( p, true, true );
+    return "Invalid UTF-8 data in block";
+    }
+  Basic_buffer tmp;
+  Point p = tmp.bof();
+  for( unsigned i = 0; i < out.size(); ++i )
+    tmp.pputc( p, out[i] );
+  replace_and_repaint( buffer, tmp );
+  return "Done";
   }
 
 
@@ -797,11 +963,11 @@ void Window_vector::encode_rot1347( const bool mode13 )
   }
 
 
-void Window_vector::remove_duplicate_lines( const bool back )
+const char * Window_vector::remove_duplicate_lines( const bool back )
   {
   Buffer & buffer = data[curwin_].buffer();
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
-      read_only( buffer ) ) return;
+      read_only( buffer ) ) return 0;
 
   bool change = false;
   if( back )
@@ -829,30 +995,29 @@ void Window_vector::remove_duplicate_lines( const bool back )
           buffer.pdell( p );
           }
     }
-  if( change )
+  if( !change ) return "No duplicate lines in block";
+  buffer.reset_appendable();
+  if( Block::valid() )
     {
-    buffer.reset_appendable();
-    if( Block::valid() )
-      {
-      if( back ) data[curwin_].update_points( Block::end(), true, true );
-      else data[curwin_].update_points( Block::begin(), true, true );
-      }
-    if( RC::editor_options().auto_unmark ) Block::reset();
-    Screen::repaint( &buffer );
+    if( back ) data[curwin_].update_points( Block::end(), true, true );
+    else data[curwin_].update_points( Block::begin(), true, true );
     }
-  else Screen::show_message( "No duplicate lines in block" );
+  if( RC::editor_options().auto_unmark ) Block::reset();
+  Screen::repaint( &buffer );
+  return 0;
   }
 
 
 const char * Window_vector::copyright_update()
   {
   static std::vector< std::string > history;
-  const std::string search_string( "\\<Copyright\\>\\+\\[^\\n]\\+\\+\\+\\[0-9]\\*\\<\\[^0-9]" );
+  const std::string search_string( "\\<Copyright\\>\\+\\[^\\n]\\[1-9]\\+\\+\\+\\+\\[0-9]\\*\\<\\[^0-9]" );
+  const std::string search_string2( "\\[1-9]\\+\\+\\+\\+\\[0-9]" );
 
   std::string year_string;
   const time_t tt = std::time( 0 );
   const std::tm * const t = std::localtime( &tt );
-  if( t )
+  if( t && t->tm_year >= 0 )
     {
     int year = 1900 + t->tm_year;
     while( year )
@@ -862,57 +1027,41 @@ const char * Window_vector::copyright_update()
   if( Screen::get_string( "Year to add (^C to abort): ", history ) <= 0 )
     return 0;
   year_string = history.back();
+  int new_year = 0;
   for( unsigned i = 0; i < year_string.size(); ++i )
-    if( !std::isdigit( year_string[i] ) )
-      year_string.clear();
-  if( year_string.empty() || year_string[0] == '0' ) return "Invalid year";
-  year_string.insert( 0, "\\<" ); year_string += "\\>";
-  bool ask = true, found = false, modified = false;
+    {
+    if( !std::isdigit( year_string[i] ) ) { year_string.clear(); break; }
+    new_year *= 10; new_year += year_string[i] - '0';
+    }
+  if( year_string.size() < 4 || year_string[0] == '0' ||
+      new_year < 1000 || new_year > 9999 ) return "Invalid year";
+  bool ask = true, found = false, modified = false, outdated = false;
+  bool skipped = false;
   for( int i = 0; i < Bufhandle_vector::handles(); next(), ++i )
     {
     Window & w = data[curwin_];
-    if( w.buffer().options.read_only ) continue;
+    if( w.buffer().options.read_only ) { skipped = true; continue; }
     Point p = w.buffer().bof();
     std::vector< std::string > regex_pieces;
     while( true )
       {
-      Point p1 = p;
-      if( !Regex::find( w.buffer(), p1, p, search_string, regex_pieces ) )
-        break;
+      Point pn = p;			// begin of notice
+      if( Regex::find( w.buffer(), pn, p, search_string, regex_pieces ) != 0 )
+        break;				// no more copyrights found
+      Point py = pn;			// begin of range of years
+      Point pt;
+      if( Regex::find( w.buffer(), py, pt, search_string2, regex_pieces ) != 0 ||
+          py <= pn || pt >= p )
+        continue;			// not a valid copyright notice
       found = true;
-      const Point p0 = w.buffer().bol( p1 );
-      const Point p2 = w.buffer().eol( p );
-      w.buffer().pprev( p );
-      const Point ph = p;		// copyright holder begins here
-      while( w.buffer().pprev( p ) && !w.buffer().piseow( p ) ) ;
-      Basic_buffer tmp( w.buffer(), p0, p );
-      Point pt = tmp.bof();
-      if( p1 < p && w.buffer().piseow( p ) &&
-          !Regex::find( tmp, pt, pt, year_string, regex_pieces ) )
+      const Point p0 = w.buffer().bol( pn );	// bol of notice
+      const Point ph2 = w.buffer().eol( p );	// end of copyright holder
+      w.buffer().pprev( p );			// begin of copyright holder
+      Basic_buffer tmp( w.buffer(), p0, py );	// "<prefix> Copyright <text>"
+      if( insert_new_year( w.buffer(), p0, pn, py, p, ph2, new_year, tmp ) )
         {
-        const int rmargin = w.buffer().options.rmargin();
-        pt = tmp.eof();
-        tmp.pputc( pt, ',' );
-        if( tmp.to_cursor( pt ).col + (int)year_string.size() - 4 > rmargin + 1 )
-          {
-          tmp.pputc( pt, '\n' );
-          tmp.pputb( pt, w.buffer(), p0, p1 );
-          }
-        else tmp.pputc( pt, ' ' );
-        for( unsigned i = 2; i < year_string.size() - 2; ++i )
-          tmp.pputc( pt, year_string[i] );
-        if( ph < p2 )
-          {
-          Point pt1 = pt;
-          tmp.pputb( pt, w.buffer(), ph, p2 );
-          if( tmp.to_cursor( pt ).col + 1 > rmargin + 1 )
-            {
-            tmp.pputc( pt1, '\n' );
-            tmp.pputb( pt1, w.buffer(), p0, p1 );
-            }
-          else tmp.pputc( pt1, ' ' );
-          }
-        p = p2;
+        outdated = true;
+        p = ph2;
         w.update_points( p, true, true );
         int key = 0;
         if( ask || Screen::kbhit() >= 0 )
@@ -924,16 +1073,27 @@ const char * Window_vector::copyright_update()
         if( !ask || key == 'y' || key == 'Y' )
           {
           w.buffer().replace( p0, p, tmp, tmp.bof(), tmp.eof() );
-          w.update_points( p );
+          w.update_points( p, true, true );
           Screen::repaint( &w.buffer() );
           modified = true;
           }
         }
       }
     }
-  if( !found ) return "No copyright notices found";
-  if( !modified ) return "No copyright notices have been modified";
-  return "Finished";
+  if( skipped )
+    {
+    if( !found ) return "No copyright notices found (read-only buffers were skipped)";
+    if( !outdated ) return "All copyright notices are up to date (read-only buffers were skipped)";
+    if( !modified ) return "No copyright notices have been modified (read-only buffers were skipped)";
+    return "Finished (read-only buffers were skipped)";
+    }
+  else
+    {
+    if( !found ) return "No copyright notices found";
+    if( !outdated ) return "All copyright notices are up to date";
+    if( !modified ) return "No copyright notices have been modified";
+    return "Finished";
+    }
   }
 
 
@@ -981,9 +1141,11 @@ const char * Window_vector::search( const int abort_key, const bool again,
     if( size <= 0 )
       {
       if( s_ini.size() && search_history.size() ) search_history.pop_back();
+      if( search_history.empty() ) search_string.clear();
       return 0;
       }
-    search_string = search_history.back(); ISO_8859::deescapize( search_string );
+    search_string = search_history.back();
+    ISO_8859::deescapize_search_string( search_string );
     std::string prompt;
     if( RC::editor_options().ignore_case )
       prompt = "(N)o_ignore_case (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
@@ -1019,7 +1181,6 @@ const char * Window_vector::search( const int abort_key, const bool again,
       }
     }
   if( toggle ) backward = !backward;
-  Screen::show_feedback( "searching..." );
 
   Point p = curwin().pointer();			// search point
   if( block )
@@ -1028,17 +1189,25 @@ const char * Window_vector::search( const int abort_key, const bool again,
     if( !again || !Block::in_block_or_end( curwin().buffer(), p ) )
       { if( backward ) p = Block::end(); else p = Block::begin(); }
     }
+
+  const Buffer * prev_replace = 0;
+  const char * msg = 0;
   bool found = false, ask = true, found_any = false, wrapped = false;
-  bool empty_replace = false;
+  bool empty_replace = false, feedback = true;
   const int handles = global ? Bufhandle_vector::handles() : 1;
   for( int i = handles - 1; i >= 0 && !found; --i )
     {
+    if( feedback )
+      { feedback = false;
+        Screen::show_feedback( ask ? "searching..." : "replacing..." ); }
     Window & w = data[curwin_];
     std::vector< std::string > regex_pieces;
     if( replace && empty_replace && !backward ) w.buffer().pnext( p );
     Point p1 = p, p2 = p;
-    found = Regex::find( w.buffer(), p1, p2, search_string, regex_pieces,
-                         icase, backward );
+    { const int tmp = Regex::find( w.buffer(), p1, p2, search_string,
+                                   regex_pieces, icase, backward );
+    if( tmp == 2 ) { msg = "Search error: check syntax"; break; }
+    found = ( tmp == 0 ); }
     empty_replace = ( found && ( p1 == p2 ) );
     if( found && block &&
         ( p1 < Block::begin() || p1 >= Block::end() || p2 > Block::end() ) )
@@ -1052,7 +1221,6 @@ const char * Window_vector::search( const int abort_key, const bool again,
         { ++i; found = false; continue; }
       found_any = true;
       if( !replace ) { w.update_points( p, true, true ); break; }
-      std::string error;
       Block::save_block_position(); w.update_points( p, false, true );
       if( ask )
         {
@@ -1067,33 +1235,45 @@ const char * Window_vector::search( const int abort_key, const bool again,
         if( ask || Screen::kbhit() >= 0 )
           {
           key = Screen::show_message( "Replace (Y)es (N)o (R)est (^C to abort): ", true );
-          if( key == 3 ) break;
+          if( key == 3 ) { if( !ask ) msg = "Interrupted"; break; }
           if( ask )
-            { if( key == 'n' || key == 'N' ) { found = false; break; }
-              if( key == 'r' || key == 'R' ) ask = false; }
+            {
+            if( key == 'n' || key == 'N' ) { found = false; break; }
+            if( key == 'r' || key == 'R' ) ask = false;
+            }
           }
         if( !ask || key == 'y' || key == 'Y' )
           {
           Block::restore_block_position();
-          if( ask ) w.buffer().reset_appendable();
-          const bool b = Regex::replace( w.buffer(), p1, p2, replace_string, regex_pieces );
-          if( ask ) w.buffer().reset_appendable();
+          if( !ask && prev_replace == &w.buffer() ) w.buffer().force_append();
+          const bool done =
+            Regex::replace( w.buffer(), p1, p2, replace_string, regex_pieces );
           Block::save_block_position();
-          if( b )
-            { if( empty_replace && ( p1 < p2 ) ) empty_replace = false;
-              found = false; if( backward ) p = p1; else p = p2;
-              w.update_points( p, false, true ); }
-          else if( w.buffer().options.read_only ) error = "Read only";
-          else error = "Replace error: check syntax";
+          if( done )
+            {
+            if( !ask ) prev_replace = &w.buffer();
+            if( empty_replace && ( p1 < p2 ) ) empty_replace = false;
+            found = false; if( backward ) p = p1; else p = p2;
+            w.update_points( p, false, true );
+            }
+          else
+            {
+            prev_replace = 0; w.buffer().reset_appendable();
+            if( w.buffer().options.read_only ) msg = "Read only";
+            else msg = "Replace error: check syntax";
+            }
           break;
           }
         }
       ++i; Block::restore_block_position();
       if( ask || found ) Screen::repaint( &w.buffer() );
-      if( error.size() ) Screen::show_message( error );
+      feedback = !block;
       continue;
       }
-    if( !ask ) w.repaint();
+
+    prev_replace = 0; w.buffer().reset_appendable();
+    if( !ask && ( block || wrapped || !RC::editor_options().search_wrap ) )
+      Screen::repaint( &w.buffer() );
     if( block ) break;
     if( !wrapped )
       {
@@ -1111,11 +1291,16 @@ const char * Window_vector::search( const int abort_key, const bool again,
         }
       }
     }
-  if( !found_any )
+  data.back().repaint();			// remove feedback
+  if( !msg )
     {
-    if( block ) return "Not found (search restricted to marked block)";
-    else return "Not found";
+    if( !found_any )
+      {
+      if( block ) msg = "Not found (search restricted to marked block)";
+      else msg = "Not found";
+      }
+    else if( wrapped ) msg = "Wrapped";
+    else if( !ask ) msg = "Done";
     }
-  else if( wrapped ) return "Wrapped";
-  return 0;
+  return msg;
   }
