@@ -1,10 +1,9 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-    2014 Antonio Diaz Diaz.
+    Copyright (C) 2005-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -22,10 +21,10 @@ int init();
 int load( int new_size = 0 );
 
 void bufhandle_menu( const int abort_key );
-void change_buffer_name();
-void load_file( const int abort_key );
+const char * change_buffer_name();
+const char * load_file( const int abort_key );
 void split_window();
-void save_file( const int abort_key );
+const char * save_file( const int abort_key );
 int close( bool abort, const bool quiet = false );
 int close_and_exit( const bool abort );
 
@@ -36,28 +35,31 @@ void prev();
 Window & window( int i );
 int windows();
 
-void add_char( const int key, const bool force = false );
+const char * add_char( const int key, const bool force = false );
 void delete_char( const bool back = false );
 void delete_line();
 void copy_block();
 void delete_block();
 void move_block();
-void read_block();
-void write_block();
+const char * read_block();
+const char * write_block();
 void indent_block();
-void unindent_block();
-void undo();
-void redo();
+const char * unindent_block();
+const char * undo();
+const char * redo();
 
 void center_line();
-void reformat();
-void change_case( const bool to_caps );
+const char * reformat();
+const char * change_case( const bool to_caps );
 void encode_base64();
-void decode_base64();
-void encode_utf8();
-void decode_quoted_printable_utf8( const bool mode_q );
+const char * decode_base64();
+bool encode_ascii_utf8( const char ** const msgp,
+                        const bool mode_a, const bool force = false );
+const char * decode_quoted_printable_utf8( const bool mode_q,
+                                           const bool force = false );
+const char * remove_utf8_out_of_range();
 void encode_rot1347( const bool mode13 );
-void remove_duplicate_lines( const bool back = false );
+const char * remove_duplicate_lines( const bool back = false );
 
 const char * copyright_update();
 const char * search_word( const int abort_key );
